#include "clientsocket.h"
#include "unit.h"
#include <QtWebSockets/QWebSocket>
#include <QJsonParseError>
#include <QJsonDocument>
#include <QJsonObject>
#include <QJsonValue>
#include <QJsonArray>
#include <QHttpMultiPart>
#include <QDebug>
#include <QFile>
#include <QHttpMultiPart>
#include <QHttpPart>
#include <QFileInfo>
#include <QDir>
#include <QStandardPaths>
#include <QDBusReply>
#include <QDBusInterface>
#include "devicetree.h"
#include "dsysinfo.h"

DCORE_USE_NAMESPACE

#define SERVER_TIMEOUT (2*1000)//post无返回的
#define EM_NETOK 70

DNetState* DNetState::g_pInstance = nullptr;
DNetState::DNetState()
{
    netInterface = new QDBusInterface("org.freedesktop.NetworkManager",
                              "/org/freedesktop/NetworkManager",
                              "org.freedesktop.NetworkManager",
                              QDBusConnection::systemBus());
    connect(netInterface, SIGNAL(StateChanged(uint)),
            this, SLOT(noticeNetState(uint)));

    int num = netInterface->property("State").toInt();

    if(num == EM_NETOK)
    {
        m_bNetState= true;
    }
    else {
        m_bNetState = false;
    }
}

DNetState::~DNetState()
{

}

DNetState* DNetState::getInstance()
{
    if(g_pInstance == nullptr){
          g_pInstance = new DNetState;
    }
    return g_pInstance;
}


void DNetState::noticeNetState(uint num)
{
    if(num == EM_NETOK)
    {
        //网络正常
        m_bNetState = true;
        emit sigNoticeNetWork(m_bNetState);
    }
    else
    {
        m_bNetState = false;
        emit sigNoticeNetWork(m_bNetState);
    }
}

bool DNetState::getNetState()
{
    return m_bNetState;
}

DSocketClient::DSocketClient(QObject* parent):
    QObject(parent),
    m_pWebSocket(nullptr),
    connectStatus(false)
{
    //获得在线客服服务器地址
    m_WebServerUrl.append("ws://").append(ServiceInfoConfig::getInstance()->getServiceUrl()).append(":").append(ServiceInfoConfig::getInstance()->getServicePort()).append("/im/v1/chat");

    //获得文件发送的服务器地址
    m_HttpUrl.append("http://").append(ServiceInfoConfig::getInstance()->getServiceUrl()).append(":").append(ServiceInfoConfig::getInstance()->getServicePort());

    m_pPostTimer = new QTimer();
    m_pPostTimer->setSingleShot(true);
    connect(m_pPostTimer,&QTimer::timeout,this,&DSocketClient::sigPostFaild);

    m_timer = new QTimer();

    m_httpRequest = new QNetworkRequest();

    m_pNetManager = new QNetworkAccessManager(this);


    connect(m_pNetManager, SIGNAL(finished(QNetworkReply*)), this, SLOT(onRequestFinished(QNetworkReply*)));

}

DSocketClient::~DSocketClient()
{
}

void DSocketClient::reConnect()          /*-<周期重连函数 */
{
    m_pWebSocket->abort();
    m_pWebSocket->open(QUrl(m_WebServerUrl));
}

/*socket建立成功后，触发该函数 */
void DSocketClient::onConnected()
{
    connectStatus = true;

    emit sigSocketCreated();

    Q_EMIT sigConnectStatus(ConnectedHost);

    m_timer->stop();
}


/* 收到Sev端的数据时，触发该函数 */
void DSocketClient::onTextReceived(QString msg)
{
    if(msg.isEmpty())
    {
        return;
    }

    QByteArray data = msg.toLocal8Bit();

    QJsonParseError json_error;
    QJsonDocument jsonDoc(QJsonDocument::fromJson(data.simplified(), &json_error));

    if(json_error.error != QJsonParseError::NoError)
    {
        qDebug() << "json error!";
    }

    QJsonObject jsonObject = jsonDoc.object();

    QStringList keys = jsonObject.keys();

    //返回数据类型
    if(keys.contains("customer_token"))  //客服发送给客户消息
    {
        emit sigRecvMessage(jsonDoc.toJson().simplified());
    }
    else if(keys.contains("message_id"))//客服返回的确认消息
    {
        QString id = jsonObject["message_id"].toString();
        if(id != "1")
        {
            emit sigRecvMessageRet(jsonDoc.toJson().simplified());
        }
    }
}

/* socket连接断开后，触发该函数 */
void DSocketClient::onDisConnected()
{
    connectStatus = false;
    m_timer->start(3000);
}

void DSocketClient::onCloseSocket()
{
    if(m_pWebSocket != nullptr)
    {
        m_pWebSocket->abort();
        m_pWebSocket->deleteLater();
        m_pWebSocket = nullptr;
    }
}

void DSocketClient::onCreateSocket(QString custom)
{
    if(m_pWebSocket == nullptr)
    {
        m_pWebSocket = new QWebSocket;

        QObject::connect(m_pWebSocket,&QWebSocket::disconnected, this, &DSocketClient::onDisConnected);
        QObject::connect(m_pWebSocket,&QWebSocket::textMessageReceived,this, &DSocketClient::onTextReceived);
        QObject::connect(m_pWebSocket,&QWebSocket::connected, this, &DSocketClient::onConnected);

        QString weburl = m_WebServerUrl + "?customer_token=" + custom;

        m_pWebSocket->open(QUrl(weburl));
    }
}

//发送消息，格式参考udesk消息定义
void DSocketClient::onSendSession(QByteArray data)
{
    //sigStartSession
    QString timestr = QString("%1").arg(QDateTime::currentSecsSinceEpoch());

    QString url =  m_HttpUrl + "/im/v1/session" + "?uname=" + getUserName()
               + "&timestamp=" + timestr + "&sign=" + getSign(timestr);

    QNetworkRequest request;
    request.setHeader(QNetworkRequest::ContentTypeHeader,QVariant("application/json"));
    request.setUrl(QUrl(url));


    m_pNetManager->post(request, data);
}


//数字签名根据时间，邮箱等组合加密而成
QString DSocketClient::getSign(QString timestr)
{
    if(timestr.isEmpty())
        return "";

    QString str =  getUserName() + "&serviceudeskapitoken" + "&" + timestr;

    QByteArray sign =  QCryptographicHash::hash(str.toLatin1(), QCryptographicHash::Sha256).toHex();

    return sign;
}

//websocket 接口- 发送用户满意度
void DSocketClient::onSendSurveyOption(QByteArray data)
{
    m_pWebSocket->sendTextMessage(QString(data));
}

//websocket接口-发送消息，格式参考udesk消息定义
void DSocketClient::onSendMessge(QByteArray data)
{
    if(m_pWebSocket == nullptr)
        return;

    if(m_pWebSocket->isValid())
    {
        m_pWebSocket->sendTextMessage(QString(data));
    }
}

void DSocketClient::onUploadFile(QString filepath, QString msgid)
{
    if(filepath.isEmpty())
        return ;

    QFileInfo info(filepath);

    QFile *file = new QFile(filepath);
    if (!file->open(QIODevice::ReadOnly))
    {
        file->close();
        return ;
    }

    QHttpMultiPart *multiPart = new QHttpMultiPart(QHttpMultiPart::FormDataType);

    //上传附件
    QHttpPart imagePart;
    imagePart.setHeader(QNetworkRequest::ContentDispositionHeader, QVariant("form-data; name=\"uploadfile\"; filename=\""+info.fileName()+"\""));
    imagePart.setBodyDevice(file);

    QHttpPart textPart;
    textPart.setHeader(QNetworkRequest::ContentTypeHeader, QVariant("text/plain"));
    textPart.setHeader(QNetworkRequest::ContentDispositionHeader, QVariant("form-data; name=\"message_id\""));
    textPart.setBody(msgid.toUtf8());

    multiPart->append(imagePart);
    multiPart->append(textPart);

    QString timestr = QString("%1").arg(QDateTime::currentSecsSinceEpoch());

    QString url =  m_HttpUrl + "/im/v1/uploadfile" + "?uname=" + getUserName()
               + "&timestamp=" + timestr + "&sign=" + getSign(timestr);


    m_httpRequest->setUrl(QUrl(url));

    QNetworkReply* reply = m_pNetManager->post(*m_httpRequest, multiPart);

    //更新上传进度
    QObject::connect(reply, &QNetworkReply::readyRead, [=]() {

        //文件关闭很多种，但是切记不能提前关闭，否则上传进度卡死。
        file->close();

    });

    return ;
}


void DSocketClient::onRequestFinished(QNetworkReply* reply)
{
    if(reply->error() == QNetworkReply::NoError) {
        //关闭定时器
        if(m_pPostTimer->isActive())
            m_pPostTimer->stop();

        QByteArray data = reply->readAll();

        //内容不为空，判断内容的返回
        if(!(data.isEmpty()))
        {
            QString subpath = reply->url().fileName();

            QJsonParseError json_error;
            QJsonDocument jsonDoc(QJsonDocument::fromJson(data.simplified(), &json_error));

            if(json_error.error != QJsonParseError::NoError)
            {
                qDebug() << "json error!";
            }

            QJsonObject jsonObject = jsonDoc.object();

            //返回内容成功
            double code  = jsonObject["code"].toDouble();

            if(subpath== "session")
            {
                if(code == 1000)
                {
                    //QJsonObject assign_info  = jsonObject["assign_info"].toObject();
                    emit sigSessionStart(QJsonDocument(jsonObject).toJson());
                }
                else if(code != 1000)
                {
                    QString statusString = jsonObject["message"].toString();

                    emit sigSessionFault(code, statusString);
                }

            }
            else if(subpath == "uploadfile")
            {
                 emit sigUploadFileRet(m_HttpUrl + "/im/v1/dl",data);
            }
            else if(subpath == "createticker")
            {
                qint64 n_tickId = jsonObject["ticket_id"].toInt();
                //工单创建成功
                emit sigTicketComplete(n_tickId);

            }
            if(subpath == "getDeviceTree")
            {
                QJsonObject obj = jsonObject["value"].toObject();
                devicetree::getInstance()->onPraseJsonArray(QJsonDocument(obj).toJson().simplified());
            }
            else if(subpath == "agentinfo")
            {
                  QJsonObject obj = jsonObject["value"].toObject();
                  emit sigInitCustomerInfo(QJsonDocument(obj).toJson().simplified());
            }
            else if(subpath == "serviceUrl")
            {
                QString url = jsonObject["value"].toString();
                //介绍和自助支持更多的网页链接
                emit sigInitSupportWebUrl(url);
            }
        }
        else
        {
            qDebug() << "data empty";
        }
    }
    else
    {
       qDebug() << "server back error";
    }

    reply->deleteLater();

}

QString DSocketClient::getUserName()
{
    QDBusInterface manual("com.deepin.daemon.Accounts",
                             "/com/deepin/daemon/Accounts/User1000",
                             "com.deepin.daemon.Accounts.User",
                             QDBusConnection::systemBus());
    QString name = manual.property("UserName").toString();
    return name;
}

QString DSocketClient::getSendUrInfo()
{
    QString timestr = QString("%1").arg(QDateTime::currentSecsSinceEpoch());
    QString strUrl = "uname=" + getUserName() + "&timestamp=" + timestr +
            "&sign=" + getSign(timestr);

    return strUrl;
}

//发送工单信息,由留言咨询调用
void DSocketClient::sendTicket(const QString username, const QByteArray tickerValue, const QByteArray categoryValue, QStringList strFilePath)
{
    QString timestr = QString("%1").arg(QDateTime::currentSecsSinceEpoch());

    QString strUrl = "uname=" + username + "&timestamp=" + QString("%1").arg(QDateTime::currentSecsSinceEpoch()) +
            "&sign=" + getSign(timestr);

    QHttpMultiPart *multiPart = new QHttpMultiPart(QHttpMultiPart::FormDataType);
     //工单基础信息
    QHttpPart ticker_Part;
    ticker_Part.setHeader(QNetworkRequest::ContentDispositionHeader, QVariant("form-data; name=\"ticker\""));
    ticker_Part.setBody(tickerValue);
    multiPart->append(ticker_Part);
    //故障类别
    QHttpPart textCategory;
    textCategory.setHeader(QNetworkRequest::ContentDispositionHeader, QVariant("form-data; name=\"category\""));
    textCategory.setBody(categoryValue);
    multiPart->append(textCategory);

     //上传附件
     for(int index = 0; index < strFilePath.count(); index++)
     {

         QHttpPart imagePart;
         imagePart.setHeader(QNetworkRequest::ContentTypeHeader, QVariant("image/jpeg"));
         QString strFileName = " ";
         int index_1 = strFilePath.at(index).lastIndexOf('/');
         strFileName = strFilePath.at(index);
         strFileName.replace(strFilePath.at(index).left(index_1+1),"");

         imagePart.setHeader(QNetworkRequest::ContentDispositionHeader, QVariant("form-data; name=\"uploadfiles\"; filename=\""+strFileName+"\""));

         QFile *file = new QFile(strFilePath.at(index));

         if(!file->open(QIODevice::ReadOnly))
         {
            qDebug()<<"open file failed";
            file->deleteLater();
            continue;
         }

        imagePart.setBodyDevice(file);
         multiPart->append(imagePart);
     }

    QString strPostUrl = m_HttpUrl + "/api/createticker?" + strUrl;

    m_httpRequest->setUrl(QString(strPostUrl));

    QNetworkReply *reply = m_pNetManager->post(*m_httpRequest, multiPart);

    if(m_pPostTimer->isActive())
        m_pPostTimer->stop();
    m_pPostTimer->start(SERVER_TIMEOUT);



    //更新上传进度
//    QObject::connect(reply, &QNetworkReply::uploadProgress, [](int a, int b) {
//        qDebug()<<"upload ..."<<a<<b<<QString::number(((float)a) / b);
//    });
    QObject::connect(reply, &QNetworkReply::uploadProgress, this, &DSocketClient::updateUploadProcess);
}
void DSocketClient::updateUploadProcess(int a, int b)
{

     emit sigUpdateProgress(QString::number(((float)a) / b));
     //有东西上传证明，不用做超时处理(提交失败)
     if((a != 0) && (b != 0))
     {
         if(m_pPostTimer->isActive())
           m_pPostTimer->stop();
     }
}

//获取页面地址
void DSocketClient::getSupportWebUrl()
{
    QString strPostUrl = m_HttpUrl + "/api/serviceUrl?" + getSendUrInfo();
    m_httpRequest->setUrl(QString(strPostUrl));
    m_pNetManager->get(*m_httpRequest);
}


//联系我们 传递内容，为json串
void DSocketClient::searchCustomerInfo(QByteArray byteArray)
{
    QString strPostUrl = m_HttpUrl + "/api/agentinfo?" + getSendUrInfo();
    m_httpRequest->setUrl(QString(strPostUrl));

    m_pNetManager->post(*m_httpRequest, byteArray);

}

//获取故障树列表
void DSocketClient::getDeviceTree()
{
    QString strPostUrl = m_HttpUrl + "/api/getDeviceTree?" + getSendUrInfo();
    m_httpRequest->setUrl(QString(strPostUrl));
    m_pNetManager->get(*m_httpRequest);
}
