#ifndef CLIENTSOCKET_H
#define CLIENTSOCKET_H

#include <QObject>

#include <QString>
#include <QTimer>
#include <QNetworkConfigurationManager>

#include <QNetworkAccessManager>    //加载网络请求头文件
#include <QNetworkReply>
#include <QDBusInterface>


class DNetState:public QObject
{
    Q_OBJECT
private:
    DNetState();
    ~DNetState();
    static DNetState* g_pInstance;
    bool m_bNetState;
public:
    static DNetState* getInstance();
private:
    QDBusInterface *netInterface;

signals:
    void sigNoticeNetWork(bool);
private slots:
    void noticeNetState(uint);
public:
    //网络改变触发的槽函数
    bool getNetState();

};

//websocket通信类
class QWebSocket;

class DSocketClient : public QObject
{
    Q_OBJECT
public:
    DSocketClient(QObject *parent=0);

    ~DSocketClient();


private:
    QString getSendUrInfo();

    QString getUserName();

private:
    QTimer *m_pPostTimer;//post定时器

    QWebSocket *m_pWebSocket;     /*-<websocket类 */

    bool connectStatus;         /*-<websocket连接状态，连接成功：true；断开：false */

    QTimer *m_timer;            /*-<周期重连Timer */

    QString m_WebServerUrl;

    QString m_HttpUrl;

    QNetworkAccessManager* m_pNetManager;

    QNetworkRequest* m_httpRequest;

private:
    void reConnect();           /*-<周期重连函数 */

    //数字签名根据时间，用户名等组合加密而成
    QString getSign(QString timestr);

private slots:
    void onConnected();                 //websocket 连接成功

    void onTextReceived(QString msg);  //在线客服聊天信息回传

    void onDisConnected();             //websocket断开连接回传

    void onRequestFinished(QNetworkReply* reply);

    void updateUploadProcess(int, int);

    //websoket 对外暴露的业务接口
public slots:

    void onCreateSocket(QString);

    void onCloseSocket();

    //发送消息，格式参考udesk消息定义
    void onSendMessge(QByteArray);

    //发送消息，格式参考udesk消息定义
    void onSendSession(QByteArray);

    //发送满意度调查
    void onSendSurveyOption(QByteArray);

    //发送文件接口
    void onUploadFile(QString filepath, QString msgid);

    //发送工单信息,有留言咨询调用
    void sendTicket(const QString username, const QByteArray tickerValue, const QByteArray categoryValue, QStringList strFilePath);


    //获取页面地址
    void getSupportWebUrl();

    //获取故障树列表
    void getDeviceTree();

    //联系我们 传递内容，为json串
    void searchCustomerInfo(QByteArray byteArray);

//通知信号。向外部模块发送服务端回传数据
signals:
    void sigSocketCreated();

    //客户端的连接状态
    void sigConnectStatus(const quint8 &state);

    //接收客服消息
    void sigRecvMessage(QByteArray);

    //会话创建完成
    void sigSessionStart(QByteArray);

    //会话创建失败返回信号
    void sigSessionFault(int, QString);

    //上传文件回传
    void sigUploadFileRet(QString, QByteArray);

    //关闭会话
    void sigSessionClose(QByteArray);

    //用户消息回传确认
    void sigRecvMessageRet(QByteArray);

    //接收用户信息回传
    void sigInitCustomerInfo(QByteArray);

    //接收网址列表回传
    void sigInitSupportWebUrl(QString);

    //工单创建成功
    void sigTicketComplete(float);
    //更新附件上传进度
    void sigUpdateProgress(QString);
    //服务器断网或服务器失联的信号
    void sigPostFaild();
};

#endif // CLIENTSOCKET_H
