#include "devicetree.h"
#include <QJsonParseError>
#include <QJsonDocument>
#include <QJsonArray>
#include <QJsonObject>
#include <QFile>
#include <QFileInfo>
#include <QDir>
#include <QStandardPaths>
#include <QDebug>
#include <QNetworkAccessManager>
#include <QNetworkRequest>
#include <QEventLoop>
#include <QNetworkReply>
#include <QCoreApplication>

devicetree* devicetree::_instance = nullptr;

devicetree* devicetree::getInstance()
{
    if(_instance == nullptr){
          _instance = new devicetree;
    }
    return _instance;
}

devicetree::devicetree()
{
}

devicetree::~devicetree()
{
    if (_instance != nullptr)
    {
        delete _instance;
        _instance = nullptr;
    }
    if(m_pCustomers.count() > 0)
    {
        //如果有用户信息则写文件
        writeCustomJsonFile();
    }
}

// 获取所有的省份
QStringList devicetree::getSystemNames()
{
    return m_pSystem;
}

// 获取某个省下的所有的城市
QStringList devicetree::getBlockNames(QString system)
{
    return m_pSystemMap.value(system);
}

// 获取某个省下的某个市下的所有的县
QStringList devicetree::getItemNames(QString block)
{
    return m_pBlockMap.value(block);
}

ServiceCustomer devicetree::getCustomService(QString system, QString block, QString item)
{
    QString sindex = system+"-"+block+"-" + item;

    return m_pCustomers.value(sindex);
}


//写json文件
void devicetree::writeDeviceJsonFile(QByteArray data)
{
    QString oldpath = QDir::currentPath();

    QString savename = "uos-devicetree.json";

    QString savepath = "/usr/share/" + QCoreApplication::applicationName() + "/ini/";

    QDir iniDir;

    if(!iniDir.exists(savepath))
    {
        iniDir.mkpath(savepath);
    }

    //将程序的执行路径设置到filePath下
    iniDir.setCurrent(savepath);

    //读写json设备文件
    QFile file(savename);

    if(!file.open(QIODevice::WriteOnly | QIODevice::Text | QFile::Truncate))
    {
       qDebug() << "File open error";
    }
    else
    {
       qDebug() <<"File open!";
    }

    file.write(data);

    file.close();
}


void devicetree::loadCustomJsonFile()
{
    QString oldpath = QDir::currentPath();

    QString savename = "customer.json";

    QString savepath = "/usr/share/" + QCoreApplication::applicationName() + "/ini/";

    QDir iniDir;

    if(!iniDir.exists(savepath))
    {
        iniDir.mkpath(savepath);
    }

    //将程序的执行路径设置到filePath下
    iniDir.setCurrent(savepath);

    QFile readfile(savename);

    if(!readfile.open(QIODevice::ReadOnly))
    {
        qDebug()<<tr("open file fail");
    }

    //每行读取文件信息，解析之后，构造customer-map;
    while(!readfile.atEnd()){
        QByteArray line = readfile.readLine();

        onPraseCustomerJsonArray(line);
    }

    readfile.close();

    QDir::setCurrent(oldpath);

    return ;
}

void devicetree::writeCustomJsonFile()
{
    QString oldpath = QDir::currentPath();

    QString savename = "customer.json";

    QString savepath = "/usr/share/" + QCoreApplication::applicationName() + "/ini/";
    QDir iniDir;

    if(!iniDir.exists(savepath))
    {
        iniDir.mkpath(savepath);
    }

    //将程序的执行路径设置到filePath下
    iniDir.setCurrent(savepath);

    QFile writefile(savename);

    if(!writefile.open(QIODevice::WriteOnly | QIODevice::Text | QFile::Truncate))
    {
        qDebug()<<tr("open file fail");
    }

    QJsonObject rootObj;

    QMapIterator<QString, ServiceCustomer> i(m_pCustomers);
    while (i.hasNext()) {
      i.next();

      ServiceCustomer customer=  i.value();
      QJsonObject obj;
      rootObj.insert("key", i.key());

      obj.insert("name",customer.name);
      obj.insert("team",customer.team);
      obj.insert("moblie",customer.telephone);
      obj.insert("email",customer.email);
      obj.insert("qrcode",customer.QRCode);

      rootObj.insert("ServiceCustomer", i.key());

      QByteArray data = QJsonDocument(rootObj).toJson().simplified();
      data.push_back("\n");

      writefile.write(data);
    }

    writefile.close();

    QDir::setCurrent(oldpath);

    return;
}

//解析文件中的没一行信息
void devicetree::onPraseCustomerJsonArray(QByteArray data)
{
    QJsonParseError json_error;
    QJsonDocument jsonDoc(QJsonDocument::fromJson(data.simplified(), &json_error));

    if(json_error.error != QJsonParseError::NoError)
    {
        qDebug() << "json error!";
    }

    QJsonObject rootObj = jsonDoc.object();



    QString key = rootObj["key"].toString();

    QJsonObject jsonObj = rootObj["ServiceCustomer"].toObject();

    ServiceCustomer customer;

    customer.name = jsonObj["name"].toString();
    customer.team = jsonObj["team"].toString();
    customer.telephone = jsonObj["moblie"].toString();
    customer.email = jsonObj["email"].toString();
    customer.QRCode = jsonObj["qrcode"].toString();

    m_pCustomers.insert(key, customer);

    return;
}

void devicetree::loadDeviceJsonFile()
{
    QString oldpath = QDir::currentPath();

    QString savename = "uos-devicetree.json";

    QString savepath = "/usr/share/" + QCoreApplication::applicationName() + "/ini/";

//    if(!iniDir.exists(savepath))
//    {
//        iniDir.mkpath(savepath);
//    }

    //将程序的执行路径设置到filePath下
    QDir::setCurrent(savepath);

    QFile readfile(savename);

    if(!readfile.open(QIODevice::ReadOnly))
    {
        qDebug()<<tr("open file fail");
    }

    QByteArray allData = readfile.readAll();

    readfile.close();

    QDir::setCurrent(oldpath);

    onPraseJsonArray(allData);

    return;
}

QString devicetree::downQRCode(QString urlpath)
{
    QString serviceurl;

    serviceurl.append("http://").append(ServiceInfoConfig::getInstance()->getServiceUrl()).append(":").append(ServiceInfoConfig::getInstance()->getServicePort());

    urlpath = serviceurl + "/" + urlpath;

    QString oldpath = QDir::currentPath();

    QFileInfo info(urlpath);

    QString savename = info.fileName();

    QString saveDir = QStandardPaths::writableLocation(QStandardPaths::CacheLocation);

    QDir CacheDir;

    if(!CacheDir.exists(saveDir))
    {
        CacheDir.mkpath(saveDir);
    }

    QDir::setCurrent(saveDir);

    QString savepath = saveDir + "/" + savename;

    //构造请求
    QNetworkAccessManager manager;
    QNetworkRequest request;
    request.setUrl(urlpath);
    // 发送请求
    QNetworkReply *reply = manager.get(request);
    //开启一个局部的事件循环，等待响应结束，退出
    QEventLoop loop;
    connect(reply, SIGNAL(finished()), &loop, SLOT(quit()));
    loop.exec();

    //判断是否出错,出错则结束
    if (reply->error() != QNetworkReply::NoError)
    {
        return "";
    }

    //保存文件
    QFile file(savename);
    if(!file.open(QIODevice::WriteOnly))
        return "";

    file.write(reply->readAll());

    file.close();

    delete reply;

    QDir::setCurrent(oldpath);

    return savepath;
}

void devicetree::insertCustomerInfo(QString key, ServiceCustomer data)
{
    m_pCustomers.insert(key, data);
}

void devicetree::onPraseJsonArray(QByteArray data)
{
    QJsonParseError json_error;
    QJsonDocument jsonDoc(QJsonDocument::fromJson(data.simplified(), &json_error));

    if(json_error.error != QJsonParseError::NoError)
    {
        qDebug() << "json error!";
    }

    QJsonObject rootObj = jsonDoc.object();

    QJsonArray rootArray = rootObj["max_categories"].toArray();

    if(rootArray.isEmpty())
    {
        return;
    }

    m_pSystem.clear();

    m_pSystemMap.clear();

    m_pBlockMap.clear();
    if (QLocale::system().name() =="zh_CN")
    {

    for(int i = 0; i < rootArray.size(); i++)
    {
        QJsonValue value = rootArray.at(i);
        if(value.isObject())
        {
            QJsonObject sysobject = value.toObject();

            QString system = sysobject["name"].toString();
            QJsonArray blockary = sysobject["mid_cateories"].toArray();

            m_pSystem.push_back(system);

            QStringList blocklist;

            for(int j = 0; j < blockary.size(); j++)
            {
                QJsonValue blockvalue = blockary.at(j);
                if(blockvalue.isObject())
                {
                    QJsonObject blockobject = blockvalue.toObject();

                    QString block = blockobject["name"].toString();
                    QJsonArray itemArray = blockobject["min_cateories"].toArray();
                    blocklist.push_back(block);

                    QStringList itemlist;

                    for(int k = 0; k < itemArray.size(); k++)
                    {
                        QJsonValue itemvalue = itemArray.at(k);
                        if(itemvalue.isObject())
                        {
                            QJsonObject itemobject = itemvalue.toObject();
                            QString item = itemobject["name"].toString();

                                itemlist.push_back(item);
                            }
                        }
                        m_pBlockMap.insert(block, itemlist);
                    }
                }
                m_pSystemMap.insert(system, blocklist);
            }
        }
    }


    else {
            for(int i = 0; i < rootArray.size(); i++)
            {
                QJsonValue value = rootArray.at(i);
                if(value.isObject())
                {
                    QJsonObject sysobject = value.toObject();

                    QString system = sysobject["ename"].toString();
                    QJsonArray blockary = sysobject["mid_cateories"].toArray();

                    m_pSystem.push_back(system);

                    QStringList blocklist;

                    for(int j = 0; j < blockary.size(); j++)
                    {
                        QJsonValue blockvalue = blockary.at(j);
                        if(blockvalue.isObject())
                        {
                            QJsonObject blockobject = blockvalue.toObject();

                            QString block = blockobject["ename"].toString();
                            QJsonArray itemArray = blockobject["min_cateories"].toArray();
                            blocklist.push_back(block);

                            QStringList itemlist;

                            for(int k = 0; k < itemArray.size(); k++)
                            {
                                QJsonValue itemvalue = itemArray.at(k);
                                if(itemvalue.isObject())
                                {
                                    QJsonObject itemobject = itemvalue.toObject();
                                    QString item = itemobject["ename"].toString();

                                    itemlist.push_back(item);
                                }
                            }
                            m_pBlockMap.insert(block, itemlist);
                        }
                    }
                    m_pSystemMap.insert(system, blocklist);
                }
            }
        }

    emit sigUpdataDeviceTreeCombox();

    //更新完设备数变量，写设备文件
    writeDeviceJsonFile(data);
    return;
}



ServiceInfoConfig* ServiceInfoConfig::g_pInstance = nullptr;

ServiceInfoConfig::ServiceInfoConfig()
{

}
ServiceInfoConfig::~ServiceInfoConfig()
{

}

ServiceInfoConfig* ServiceInfoConfig::getInstance()
{
    if(g_pInstance == nullptr){
          g_pInstance = new ServiceInfoConfig;
    }
    return g_pInstance;
}

void ServiceInfoConfig::loadConfigJsonFile()
{
    QString oldpath = QDir::currentPath();

    QString savename = "service_support_config.json";


    QString savepath = "/usr/share/" + QCoreApplication::applicationName() + "/ini/";
    QDir iniDir;

    if(!iniDir.exists(savepath))
    {
        iniDir.mkpath(savepath);
    }

    //将程序的执行路径设置到filePath下
    iniDir.setCurrent(savepath);

    QFile loadFile(savename);

    if(!loadFile.open(QIODevice::ReadOnly))
    {
        qDebug()<<"open service config json file fail";
    }

    QByteArray allData = loadFile.readAll();

    loadFile.close();
    onPraseJsonArray(allData);

    iniDir.setCurrent(oldpath);

    return;
}


void ServiceInfoConfig::onPraseJsonArray(const QByteArray &data)
{
    QJsonParseError json_error;
    QJsonDocument jsonDoc(QJsonDocument::fromJson(data.simplified(), &json_error));

    if(json_error.error != QJsonParseError::NoError)
    {
        qDebug() << "json error!";
        return;
    }

    QJsonObject rootObj = jsonDoc.object();

    strServiceUrl = rootObj["service_url"].toString();
    strServicePort = rootObj["service_port"].toString();

    return;
}

void ServiceInfoConfig::writeJsonFile(const QByteArray &data)
{
    QString oldpath = QDir::currentPath();

    QString savename = "service_support_config.json";

    QString savepath = "/usr/share/" + QCoreApplication::applicationName() + "/ini/";;

    QDir iniDir;

    if(!iniDir.exists(savepath))
    {
        iniDir.mkpath(savepath);
    }

    //将程序的执行路径设置到filePath下
    iniDir.setCurrent(savepath);

    //读写json设备文件
    QFile file(savename);

    if(!file.open(QIODevice::WriteOnly | QIODevice::Text | QFile::Truncate))
    {
       qDebug() << "File open error";
    }
    else
    {
       qDebug() <<"File open!";
    }

    file.write(data);

    file.close();

    iniDir.setCurrent(oldpath);
}

QString ServiceInfoConfig::getServiceUrl()
{
    return strServiceUrl;
}

QString ServiceInfoConfig::getServicePort()
{
    return strServicePort;
}
