/*
 * Copyright (C) 2020 ~ 2020 Deepin Technology Co., Ltd.
 *
 * Author:     zhanglei <zhanglei_cm@deepin.com>
 *
 * Maintainer: zhanglei <zhanglei_cm@deepin.com>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <QApplication>
#include <DApplication>
#include <DMainWindow>
#include <DWidgetUtil>
#include <DGuiApplicationHelper>
#include <QTranslator>
#include <QDBusConnection>
#include <QDBusError>
#include <QDebug>
#include "mainwindow.h"
#include "servicevandsupportdbus.h"

DWIDGET_USE_NAMESPACE

int main(int argc, char *argv[])
{
    DApplication::loadDXcbPlugin();
    DApplication a(argc, argv);
    a.setAttribute(Qt::AA_UseHighDpiPixmaps);
    a.loadTranslator();
    DGuiApplicationHelper::instance()->setPaletteType(DGuiApplicationHelper::UnknownType);
  //  a.setTheme("light");
    a.setApplicationName("uos-service-support");
    a.setOrganizationName("deepin");
   // a.setApplicationVersion("1.0.3");
#ifdef CVERSION
    a.setApplicationVersion(CVERSION);
#else
    a.setApplicationVersion("1.0.9");
#endif
    a.setProductIcon(QIcon(":/images/title.svg"));
   // a.setProductName(QObject::tr("Support"));

    //单例模式
    if (!a.setSingleInstance(a.applicationName()))
    {
      exit(-1);
    }

    QTranslator translator;
    //translator.load(QString(":/qm/Translation"));
    //获得应用程序名称
    QString strAppName = QCoreApplication::applicationName();

    translator.load("/usr/share/deepin-" + strAppName + "/translations/" + strAppName + "_" + QLocale::system().name());

    a.installTranslator(&translator);

    a.setApplicationDisplayName(QObject::tr("Support"));
    a.setApplicationDescription(QObject::tr("Support helps users get official services and technical support in various manners."));

    MainWindow w;
    //dbus
    QDBusConnection connection = QDBusConnection::sessionBus();
    if(!connection.isConnected())
    {
        qDebug()<<"DBus not connect!";
    }
    if(!connection.registerService(SERVICE_NAME))
    {
        qDebug()<<"register service failed!"<<connection.lastError().message();
    }
    QDBusConnection::RegisterOptions options = QDBusConnection::ExportAllSlots | QDBusConnection::ExportAllSignals;

    if(!connection.registerObject(SERVICEANDSUPPORT_SERVICE_PATH, SERVICEANDSUPPORT_INTERFACE_NAME, &w, options))
    {
        qDebug()<<"register object failed!"<<connection.lastError().message();
    }

    w.show();

    Dtk::Widget::moveToCenter(&w);


    return a.exec();
}
