/*
 * Copyright (C) 2020 ~ 2020 Deepin Technology Co., Ltd.
 *
 * Author:     zhanglei <zhanglei_cm@deepin.com>
 *
 * Maintainer: zhanglei <zhanglei_cm@deepin.com>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "mainwindow.h"
#include <DTitlebar>
#include <QDBusInterface>
#include <QDebug>
#include <QMap>
#include <QMargins>
#include <QVariant>
#include <QDBusInterface>
#include <QDBusReply>
#include <DApplicationHelper>
#include "devicetree.h"

const QSize ListViweItemSize_ListMode(160,48);

const int WidgetMinimumWidget = 820;
const int WidgetMinimumHeight = 650;

#define TIMER_TIMEOUT   (5*1000)
MainWindow::MainWindow(QWidget *parent)
    : DMainWindow(parent)
    , m_navModel(nullptr)
   // ,m_bNetState(false)
{
    //加载服务器配置信息
    ServiceInfoConfig::getInstance()->loadConfigJsonFile();
    devicetree::getInstance()->loadDeviceJsonFile();
    setWindowIcon(QIcon(":/images/title.svg"));
    initAllModule();
    initUI();
    initSocketClient();
    connect(m_navView,SIGNAL(clicked(QModelIndex)),this,SLOT(showClick(QModelIndex)));

    connect(m_pNoNetWidget,&NoNetworkWidget::chagePage,this,
    [=]()
    {
         m_stackWidget->setCurrentWidget(m_pContentUs);
         //切左侧导航栏
         int nowlist=3;
         QModelIndex indexFromList = m_navModel->index(nowlist, 0);
         m_navView->clicked(indexFromList);
         m_navView->setCurrentIndex(indexFromList);
    });


    //提交成功界面，点击完成，返回到应用主界面（介绍页面）；
    connect(m_pSuccessWidget,&SuccessWidget::changePage,this,
    [=]()
    {
        m_stackWidget->setCurrentWidget(m_pIntroduction);
        //切左侧导航栏
        int nowlist=0;
        QModelIndex indexFromList = m_navModel->index(nowlist, 0);
        m_navView->clicked(indexFromList);
        m_navView->setCurrentIndex(indexFromList);
    });

    connect( DNetState::getInstance(), &DNetState::sigNoticeNetWork, this, &MainWindow::slotNetWorkState);

    int nowlist=0;
    QModelIndex indexFromList = m_navModel->index(nowlist, 0);
    m_navView->clicked(indexFromList);
    m_navView->setCurrentIndex(indexFromList);

}

MainWindow::~MainWindow()
{
    qDebug() << "~MainWindow";
}

void MainWindow::initAllModule()
{
    m_modules = {
        {tr("Introduction"), QIcon(":/images/introduce.svg")},
        {tr("Consults"), QIcon(":/images/message.svg")},
        {tr("Online Support"), QIcon(":/images/onlineHelp.svg")},
        {tr("Contact Us"), QIcon(":/images/connectustwo.svg")},
        {tr("Self-Service"), QIcon(":/images/auto.svg")},
       };
}

void MainWindow::initUI()
{
    QFont ft;
    ft.setPointSize(10);

    //Initialize view and layout structure
    QWidget *content = new QWidget(this);
    m_contentLayout = new QHBoxLayout(content);
    m_contentLayout->setContentsMargins(0, 0, 0, 0);
    m_contentLayout->setSpacing(0);

    ////////////////
    m_leftContentLayout = new QHBoxLayout;
    m_leftFrame = new DFrame;

    m_navView = new DListView(this);
    m_navView->setFrameShape(QFrame::Shape::NoFrame);
    m_navView->setEditTriggers(QListView::NoEditTriggers);
    m_navView->setResizeMode(QListView::Adjust);
    m_navView->setAutoScroll(true);
    m_navView->setBackgroundType(DStyledItemDelegate::BackgroundType::ClipCornerBackground);
    m_navView->setIconSize(QSize(14, 14));
    m_navView->setItemSize(ListViweItemSize_ListMode);
    DPalette pa = DApplicationHelper::instance()->palette(m_navView);
    pa.setBrush(DPalette::ItemBackground, Qt::transparent);
    DApplicationHelper::instance()->setPalette(m_navView, pa);

    QFont list_font;
    list_font.setPointSize(50);
    list_font.setBold(false);
    list_font.setWeight(20);
    m_navView->setFont(list_font);

    m_navModel = new QStandardItemModel(m_navView);

    for (auto it = m_modules.cbegin(); it != m_modules.cend(); ++it) {
            DStandardItem *item = new DStandardItem;
            item->setFont(ft);
            item->setText(it->first);
            item->setIcon(it->second);
            m_navModel->appendRow(item);
    }

    m_navView->setModel(m_navModel);

    m_leftFrame->setLayout(m_leftContentLayout);
    m_leftContentLayout->addWidget(m_navView);


    m_stackWidget = new QStackedWidget;
    m_pIntroduction = new Introduction;
    m_pMessageConsultation = new MessageConsultation;
    m_pCustomerChat = new CustomerChatWidget;
    m_pContentUs = new ContactUs;
    m_pSelf_Support = new Self_Support;
    m_pNoNetWidget = new NoNetworkWidget;
    m_pSuccessWidget = new SuccessWidget;
    m_stackWidget->addWidget(m_pIntroduction);
    m_stackWidget->addWidget(m_pMessageConsultation);
    m_stackWidget->addWidget(m_pCustomerChat);
    m_stackWidget->addWidget(m_pContentUs);
    m_stackWidget->addWidget(m_pSelf_Support);
    m_stackWidget->addWidget(m_pNoNetWidget);
    m_stackWidget->addWidget(m_pSuccessWidget);

    m_stackWidget->setCurrentWidget(m_pIntroduction);

    m_rightContentLayout = new QHBoxLayout;
    m_rightContentLayout->addWidget(m_stackWidget);
    m_rightFrame = new DFrame;
    m_rightFrame->setContentsMargins(10, 10, 10, 10);
    m_rightFrame->setLayout(m_rightContentLayout);

    m_leftFrame->setFixedWidth(180);
    m_rightFrame->setFixedWidth(630);
    m_contentLayout->setSpacing(0);
    m_contentLayout->addWidget(m_leftFrame);
    m_contentLayout->addWidget(m_rightFrame);
    setCentralWidget(content);

    //菜单
    DTitlebar *titlebar = this->titlebar();
    titlebar->setTitle("");
    auto menu = titlebar->menu();
    if (!menu) {
        qDebug() << "menu is nullptr, create menu!";
        menu = new QMenu;
    } else {
        qDebug() << "get title bar menu :" << menu;
    }
    titlebar->setMenu(menu);

//    auto action = new QAction(tr("Help"));
//    menu->addAction(action);
//    connect(action, &QAction::triggered, this, [] {
//        QDBusInterface manual("com.deepin.Manual.Open",
//                                 "/com/deepin/Manual/Open",
//                                 "com.deepin.Manual.Open",
//                                 QDBusConnection::sessionBus());
//        QDBusReply<void> reply = manual.call("Open","");
//            if (reply.isValid())  {
//                qDebug() << "call com.deepin.Manual.Open success";
//                return;
//            }
//            qDebug() << "call com.deepin.Manual.Open failed" << reply.error();
//    });
    //back
    DLabel *head_picture = new DLabel;
    DLabel *tt = new DLabel;
    tt->setFixedSize(10,10);
    head_picture->setFixedSize(32,32);
    QImage *image = new QImage;
    image->load(":/images/title.svg");
    head_picture->setPixmap(QPixmap::fromImage(*image));
    head_picture->setScaledContents(true);


//    m_backwardBtn = new DIconButton(this);
//    m_backwardBtn->setEnabled(false);
//    m_backwardBtn->setIcon(QStyle::SP_ArrowBack);

//    m_hlay->addWidget(head_picture);
//    m_hlay->addWidget(m_backwardBtn);

    titlebar->addWidget(tt, Qt::AlignLeft);
    titlebar->addWidget(head_picture, Qt::AlignLeft);
//    titlebar->addWidget(m_backwardBtn, Qt::AlignLeft);
//    titlebar->setIcon(QIcon(":/images/title.png"));
    setFixedSize(QSize(WidgetMinimumWidget, WidgetMinimumHeight));
}

void MainWindow::showClick(QModelIndex index)
{
    QString strTemp;
    strTemp = index.data().toString();

    //切任意界面，将无网络页面的倒计时内容设置为5
    if(strTemp == tr("Introduction"))
    {
        m_stackWidget->setCurrentWidget(m_pIntroduction);
    }
    else if(strTemp == tr("Consults"))
    {
        if(DNetState::getInstance()->getNetState())
        {
            m_stackWidget->setCurrentWidget(m_pMessageConsultation);
        }
        else
        {
            m_stackWidget->setCurrentWidget(m_pNoNetWidget);
            if(strRecClk != tr("Consults"))//点击的不是自己需要重置
            {
                m_pNoNetWidget->resetValue();;
            }
        }
    }
    else if(strTemp == tr("Online Support"))
    {
        //判断网络连接状态
        if(DNetState::getInstance()->getNetState())
        {
            m_stackWidget->setCurrentWidget(m_pCustomerChat);
        }
        else
        { 

            m_stackWidget->setCurrentWidget(m_pNoNetWidget);
            if(strRecClk != tr("Online Support"))//点击的不是自己需要重置
            {
                 m_pNoNetWidget->resetValue();;
            }

        }
    }
    else if(strTemp == tr("Contact Us"))
    {
        m_stackWidget->setCurrentWidget(m_pContentUs);

    }
    else if(strTemp == tr("Self-Service"))
    {
        m_stackWidget->setCurrentWidget(m_pSelf_Support);
    }
    strRecClk = strTemp;
}

void MainWindow::showWindow()
{
    this->show();
}

void MainWindow::initSocketClient()
{
    m_pWebClient = new DSocketClient();

    connect(devicetree::getInstance(), &devicetree::sigUpdataDeviceTreeCombox, m_pContentUs,&ContactUs::slotUpdateComboBox);
    connect(devicetree::getInstance(), &devicetree::sigUpdataDeviceTreeCombox, m_pMessageConsultation,&MessageConsultation::slotUpdateComboBox);
    connect(m_pMessageConsultation, &MessageConsultation::signalTicket, m_pWebClient,&DSocketClient::sendTicket);
    connect(m_pContentUs, &ContactUs::sigsearchCustomInfo, m_pWebClient,&DSocketClient::searchCustomerInfo);

    connect(m_pWebClient, &DSocketClient::sigInitCustomerInfo, m_pContentUs,&ContactUs::slotUpdateCustomInfo);
    connect(m_pWebClient, &DSocketClient::sigInitSupportWebUrl, m_pIntroduction,&Introduction::slotUpdateWebUrl);
    connect(m_pWebClient, &DSocketClient::sigInitSupportWebUrl, m_pSelf_Support,&Self_Support::slotUpdateWebUrl);

    connect(m_pWebClient, &DSocketClient::sigUpdateProgress,m_pMessageConsultation, &MessageConsultation::slotUpdateProgress);

    connect(m_pWebClient, &DSocketClient::sigTicketComplete, this,
    [=](qint64 n_tickId)
    {
        if(n_tickId != 0)
        {
            //获得留言咨询提交成功的信号后切换页面
            m_stackWidget->setCurrentWidget(m_pSuccessWidget);
        }
    });
    connect(m_pWebClient, &DSocketClient::sigTicketComplete, m_pMessageConsultation, &MessageConsultation::slotTicketComplete);

    connect(m_pWebClient, &DSocketClient::sigRecvMessage,m_pCustomerChat,&CustomerChatWidget::slotRecvMessage);
    connect(m_pWebClient, &DSocketClient::sigSessionStart,m_pCustomerChat,&CustomerChatWidget::slotSessionStart);
    connect(m_pWebClient, &DSocketClient::sigSessionFault,m_pCustomerChat,&CustomerChatWidget::slotSessionFault);

    connect(m_pWebClient, &DSocketClient::sigUploadFileRet,m_pCustomerChat, &CustomerChatWidget::slotUploadFileSuccess);
    connect(m_pWebClient, &DSocketClient::sigRecvMessageRet,m_pCustomerChat, &CustomerChatWidget::slotRecvMessageRet);
    connect(m_pCustomerChat, &CustomerChatWidget::sigStartSession, m_pWebClient, &DSocketClient::onSendSession);
    connect(m_pCustomerChat, &CustomerChatWidget::sigCreateSocket, m_pWebClient, &DSocketClient::onCreateSocket);
    connect(m_pCustomerChat, &CustomerChatWidget::sigCloseWebSocket, m_pWebClient, &DSocketClient::onCloseSocket);

    connect(m_pWebClient, &DSocketClient::sigSocketCreated, m_pCustomerChat, &CustomerChatWidget::onSocketCreated);

    connect(m_pCustomerChat, &CustomerChatWidget::signalMessage, m_pWebClient,&DSocketClient::onSendMessge);
    connect(m_pCustomerChat, &CustomerChatWidget::signalUploadFile, m_pWebClient,&DSocketClient::onUploadFile);
    connect(m_pCustomerChat, &CustomerChatWidget::signalSurvey, m_pWebClient,&DSocketClient::onSendSurveyOption);

    //申请故障树
    m_pWebClient->getDeviceTree();
    //申请url
    m_pWebClient->getSupportWebUrl();
    //post失败
    connect(m_pWebClient, &DSocketClient::sigPostFaild,m_pMessageConsultation, &MessageConsultation::slotPostFaild);
}

void MainWindow::slotNetWorkState(bool isConnected)
{
    if(isConnected)
    {
        //申请故障树
        m_pWebClient->getDeviceTree();
        //申请url
        m_pWebClient->getSupportWebUrl();
    }
}
