/*
 * Copyright (C) 2020 ~ 2020 Deepin Technology Co., Ltd.
 *
 * Author:     zhanglei <zhanglei_cm@deepin.com>
 *
 * Maintainer: zhanglei <zhanglei_cm@deepin.com>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef MAINWINDOW_H
#define MAINWINDOW_H

#include <DMainWindow>
#include <DFrame>
#include <QHBoxLayout>
#include <DBackgroundGroup>
#include <DListView>
#include <DIconButton>
#include <QStackedWidget>
#include <QStandardItemModel>
#include <DStandardItem>
#include <QModelIndex>
#include "servicevandsupportdbus.h"
#include "modules/introduction/introduction.h"
#include "modules/messageconsultation/messageconsultationwidget.h"
#include "modules/customerchat/customerchatwidget.h"
#include "modules/contactus/contactus.h"
#include "modules/self_support/self_support.h"
#include "modules/commonwidget/nonetworkwidget.h"
#include "modules/commonwidget/success.h"
#include "clientsocket.h"
#include <QNetworkConfigurationManager>

DWIDGET_USE_NAMESPACE

class MainWindow : public DMainWindow
{
    Q_CLASSINFO("D-Bus Interface", SERVICEANDSUPPORT_INTERFACE_NAME)
    Q_OBJECT
public:
    explicit MainWindow(QWidget *parent = nullptr);
    ~MainWindow();

private slots:
    void showClick(QModelIndex index);

private:
    void initUI();
    void initAllModule();
    void initSocketClient();

private:
    QHBoxLayout *m_contentLayout;

    QHBoxLayout *m_leftContentLayout;
    DFrame *m_leftFrame;
    DListView *m_navView;
    QStandardItemModel *m_navModel;

    QHBoxLayout *m_rightContentLayout;
    DFrame *m_rightFrame;
    QStackedWidget *m_stackWidget;

    DIconButton *m_backwardBtn;

private:
    QList<QPair<QString,QIcon>> m_modules;
    Introduction *m_pIntroduction;
    MessageConsultation *m_pMessageConsultation;
    CustomerChatWidget *m_pCustomerChat;
    ContactUs *m_pContentUs;
    Self_Support *m_pSelf_Support;
    NoNetworkWidget *m_pNoNetWidget;
    SuccessWidget *m_pSuccessWidget;
    QNetworkConfigurationManager *m_pNetCfgMgr;
    DSocketClient *m_pWebClient;
    //记录导航栏上次点击的内容
    QString strRecClk;

public slots:
 //给外部的接口，dbus调用使用
    void showWindow();
    void slotNetWorkState(bool isConnected);
};


#endif // MAINWINDOW_H
