/*
 * Copyright (C) 2020 ~ 2020 Deepin Technology Co., Ltd.
 *
 * Author:     zhanglei <zhanglei_cm@deepin.com>
 *
 * Maintainer: zhanglei <zhanglei_cm@deepin.com>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "nonetworkwidget.h"
#include <QDebug>

NoNetworkWidget::NoNetworkWidget(QWidget* parent )
    :DWidget(parent)
{
    initUI();

    m_TimeNum = 5;
    connect(button, &DSuggestButton::clicked, this,
            [=]()
    {
        //切换页面前将默认显示设置为5
        strLabel = tr("You may use internal network or network unavailable, and will be redirected to the Contact Us page");
        strLabel.append(tr("\nYou also can seek help in other ways."));
        strLabel.append(tr("\nCountdown ")).append("5").append(" s");

        emit chagePage();
    });
}

NoNetworkWidget::~NoNetworkWidget()
{

}


void NoNetworkWidget::resetValue()
{
    m_TimeNum = 5;
    if (ptime->isActive())
    {
        ptime->stop();
        ptime->start(1000);
        strLabel = tr("You may use internal network or network unavailable, and will be redirected to the Contact Us page");
        strLabel.append(tr("\nYou also can seek help in other ways."));
        strLabel.append(tr("\nCountdown ")).append("5").append(" s");
        lable->setText(strLabel);
    }

}

void NoNetworkWidget::initUI()
{
    QFont ft;
    ft.setPointSize(9);

    this->setFixedSize(620,580);
    head_picture = new DLabel;

    QImage *image = new QImage;
    image->load(":/images/nonetwork.svg");
    head_picture->setPixmap(QPixmap::fromImage(*image));
    head_picture->setScaledContents(true);
    DLabel *success_lable  = new DLabel;

    success_lable->setText(tr("No Network Connection"));
    success_lable->setFont(ft);
    lable  = new DLabel;

    strLabel = tr("You may use internal network or network unavailable, and will be redirected to the Contact Us page");
    strLabel.append(tr("\nYou also can seek help in other ways."));
    strLabel.append(tr("\nCountdown ")).append("5").append(" s");
    lable->setFont(ft);
    lable->setText(strLabel);
    lable->setAlignment(Qt::AlignHCenter);
    button = new DSuggestButton;
    button->setFixedSize(200,36);
    button->setText(tr("Contact Us"));
    button->setFont(ft);
    layout = new QVBoxLayout;
    layout->addSpacing(75);
    layout->addWidget(head_picture,0,Qt::AlignCenter);
    layout->addSpacing(10);
    layout->addWidget(success_lable,0,Qt::AlignCenter);
    layout->addSpacing(10);
    layout->addWidget(lable,0,Qt::AlignCenter);
    layout->addSpacing(45);
    layout->addWidget(button,0,Qt::AlignCenter);
    layout->addSpacing(213);
    this->setLayout(layout);
}

void NoNetworkWidget::setLabelText()
{
    QString time = QString("%1").arg(m_TimeNum);
    strLabel = tr("You may use internal network or network unavailable, and will be redirected to the Contact Us page");
    strLabel.append(tr("\nYou also can seek help in other ways."));
    strLabel.append(tr("\nCountdown ")).append(time).append(" s");
    lable->setText(strLabel);
}

void NoNetworkWidget::showEvent(QShowEvent *event)
{
    strLabel = tr("You may use internal network or network unavailable, and will be redirected to the Contact Us page");
    strLabel.append(tr("\nYou also can seek help in other ways."));
    strLabel.append(tr("\nCountdown ")).append("5").append(" s");
    lable->setText(strLabel);
//定时启动
    m_TimeNum = 5;
    ptime = new QTimer;
    connect(ptime, &QTimer::timeout, this, &NoNetworkWidget::slotCalTime);
    ptime->start(1000);
}

void NoNetworkWidget::hideEvent(QHideEvent *event)
{
    m_TimeNum = 5;
    ptime->stop();
    if(ptime == nullptr)
    {
        delete  ptime;
        ptime = nullptr;
    }
}

void NoNetworkWidget::slotCalTime()
{
     m_TimeNum--;

     setLabelText();

     if(m_TimeNum <= 0)
     {
         m_TimeNum = 5;
         ptime->stop();
         if(ptime == nullptr)
         {
             delete  ptime;
             ptime = nullptr;
         }
         emit chagePage();
     }
}

