#include "contactus.h"

#include <QIcon>
#include <QUrl>
#include <QPalette>
#include <QListView>
#include <QDebug>
#include <QFont>
#include <QJsonValue>
#include <QJsonArray>
#include <QJsonObject>
#include <QJsonDocument>
#include <QByteArray>
#include <QFile>
#include <QFileDialog>
#include <DPalette>
#include <QStandardPaths>
#include <QNetworkReply>
#include <QEventLoop>

#include "../../devicetree.h"

ContactUs::ContactUs(QWidget *parent) : DWidget(parent),m_curSerchKey("")
{
    //故障树结构
    DFrame* pArchFrame = createArchFrame();

    //客服名片
    DFrame* pCardFrame = createCardFrame();

    //信号槽链接
    QObject::connect(m_SystemComboBox, &QComboBox::currentTextChanged, \
                     this, &ContactUs::onSystemComboBoxIndexChanged);
    QObject::connect(m_BlockComboBox,&QComboBox::currentTextChanged, \
                     this, &ContactUs::onBlockComboBoxIndexChanged);
    QObject::connect(m_ItemComboBox, &QComboBox::currentTextChanged, \
                     this, &ContactUs::onItemComboBoxIndexChanged);

    QVBoxLayout* m_pMainLayout=new QVBoxLayout;
    m_pMainLayout->addWidget(pArchFrame);
    m_pMainLayout->addWidget(pCardFrame);

    m_pMainLayout->setSpacing(20);
    m_pMainLayout->setStretchFactor(pArchFrame,10);
    m_pMainLayout->setStretchFactor(pCardFrame,40);

    m_pMainLayout->setContentsMargins(0,0,0,50);

    this->setLayout(m_pMainLayout);

    //初始化combobox列表
    initComboBox();

    //加载用户文
    devicetree::getInstance()->loadCustomJsonFile();

}


ContactUs::~ContactUs()
{
}

DFrame* ContactUs::createCardFrame()
{

    QFont ft;
    ft.setPointSize(9);
    QPalette color_title;
    color_title.setColor(QPalette::Text,QColor(82,106,127,255));

    DFrame* pCardFrame = new DFrame;
    pCardFrame->setBackgroundRole(DPalette::ItemBackground);
    QVBoxLayout* pCardLayout = new QVBoxLayout;

    DLabel* cardLabel = new DLabel(tr("Contact Card"));
    cardLabel->setFont(ft);

    QHBoxLayout* phLayout = new QHBoxLayout;

    QVBoxLayout* puosLayout = new QVBoxLayout;
    QVBoxLayout* pInfoLayout = new QVBoxLayout;
    QVBoxLayout* pAratarLayout = new QVBoxLayout;

    QLabel* pteamlabel = new QLabel;
    pteamlabel->setPixmap(QPixmap(":/images/team.svg"));

    QLabel* ptellabel = new QLabel;
    ptellabel->setPixmap(QPixmap(":/images/phone.svg"));

    QLabel* pemaillabel = new QLabel;
    pemaillabel->setPixmap(QPixmap(":/images/mail_light.svg"));

    puosLayout->addWidget(pteamlabel);
    puosLayout->addWidget(ptellabel);
    puosLayout->addWidget(pemaillabel);

    puosLayout->setAlignment(Qt::AlignRight);
    puosLayout->setContentsMargins(0,15,0,15);

    QVBoxLayout* pLay1 = new QVBoxLayout;
    QVBoxLayout* pLay2 = new QVBoxLayout;
    QVBoxLayout* pLay3 = new QVBoxLayout;

    pLay1->setSpacing(0);pLay1->setContentsMargins(0,8,0,8);
    pLay2->setSpacing(0);pLay2->setContentsMargins(0,8,0,8);
    pLay3->setSpacing(0);pLay3->setContentsMargins(0,8,0,8);

    m_pBelLabel = new DLabel;
    m_pBelLabel->setText(tr("Service and support team"));
    m_pBelLabel->setFont(ft);
    m_pBelLabel->setPalette(color_title);

    m_pTeamLabel = new DLabel;
    m_pTeamLabel->setText(tr("UOS Support Team"));
    m_pTeamLabel->setFont(ft);

    pLay1->addWidget(m_pBelLabel);
    pLay1->addWidget(m_pTeamLabel);

    m_pTelLabel = new DLabel;
    m_pTelLabel->setText(tr("Telephone"));
    m_pTelLabel->setFont(ft);
    m_pTelLabel->setPalette(color_title);

    m_pTelTextLabel = new DLabel;
    m_pTelTextLabel->setText(tr("400-8588-488"));
    m_pTelTextLabel->setFont(ft);

    pLay2->addWidget(m_pTelLabel);
    pLay2->addWidget(m_pTelTextLabel);

    m_pEmailLabel = new DLabel;
    m_pEmailLabel->setText(tr("Email"));
    m_pEmailLabel->setFont(ft);
    m_pEmailLabel->setPalette(color_title);

    m_pEmailTextLabel = new DLabel;
    m_pEmailTextLabel->setText("support@uniontech.com");
    m_pEmailTextLabel->setFont(ft);

    pLay3->addWidget(m_pEmailLabel);
    pLay3->addWidget(m_pEmailTextLabel);

    pInfoLayout->addLayout(pLay1);
    DHorizontalLine* pline1 = new DHorizontalLine;
    pline1->setLineWidth(2);
    pInfoLayout->addWidget(pline1);
    pInfoLayout->addLayout(pLay2);

    DHorizontalLine* pline2 = new DHorizontalLine;
    pline2->setLineWidth(2);
    pInfoLayout->addWidget(pline2);
    pInfoLayout->addLayout(pLay3);

    pInfoLayout->setSpacing(2);
    pInfoLayout->setContentsMargins(0,20,0,20);

    //二维码
    m_pQRCodeLabel = new DLabel;

    m_pQRCodeLabel->setFixedSize(109, 109);
    QSize laSize = m_pQRCodeLabel->size();
    QImage image;
    image.load(":/images/contact_me_qr.png");
    QImage image1=image.scaled(laSize,Qt::IgnoreAspectRatio);//重新调整图像大小以适应label
    m_pQRCodeLabel->setPixmap(QPixmap::fromImage(image1));

    QHBoxLayout* pBottomLayout = new QHBoxLayout;
    QLabel* pCompanyWXIconLabel = new QLabel;
    QLabel* pCompanyWXTextLabel = new QLabel;
    pBottomLayout->addWidget(pCompanyWXIconLabel);

    pBottomLayout->addWidget(pCompanyWXTextLabel);

    pCompanyWXIconLabel->setPixmap(QPixmap(":/images/wechat.svg"));
    pCompanyWXIconLabel->setFixedSize(30,30);
    pCompanyWXTextLabel->setText(tr("WeChat Work"));
    pCompanyWXTextLabel->setFont(ft);
    pBottomLayout->setSpacing(11);

    pAratarLayout->addStretch(0);
    pAratarLayout->addWidget(m_pQRCodeLabel);
    pAratarLayout->setSpacing(10);
    pAratarLayout->addLayout(pBottomLayout);
    pAratarLayout->addStretch(0);

    phLayout->addLayout(puosLayout);

    DVerticalLine* pline3 = new DVerticalLine;
    pline3->setLineWidth(2);
    phLayout->addWidget(pline3);
    phLayout->addSpacing(0);
    phLayout->addLayout(pInfoLayout);
    phLayout->addSpacing(20);
    phLayout->addLayout(pAratarLayout);

    phLayout->setStretchFactor(puosLayout, 2);
    phLayout->setStretchFactor(pline3, 1);
    phLayout->setStretchFactor(pInfoLayout, 6);
    phLayout->setStretchFactor(pAratarLayout, 6);

    phLayout->setContentsMargins(3,30,0,30);
    //客服名片Frame
    DFrame *card = new DFrame;
    card->setBackgroundRole(DPalette::NoType);
    card->setLayout(phLayout);

    QHBoxLayout* playout = new QHBoxLayout;

    playout->addWidget(card);
    playout->setContentsMargins(30,10,30,10);

    pCardLayout->addWidget(cardLabel);
    pCardLayout->addLayout(playout);
    pCardFrame->setLayout(pCardLayout);


    return pCardFrame;
}

DFrame* ContactUs::createArchFrame()
{
    QFont ft;
    ft.setPointSize(9);
    QPalette color_Introduction;
    color_Introduction.setColor(QPalette::Text,QColor(82,106,127,255));

    DFrame* pArchFrame = new DFrame(this);
    pArchFrame->setBackgroundRole(DPalette::ItemBackground);

    //Combox文本介绍
    QLabel* pTextLabel = new QLabel;
    pTextLabel->setText(tr("          After selecting the category you want to consult below, the contact information of the matching support team member will be shown.You can call, email,or scan the QR code of WeChat Work from the team member via your personal WeChat, add him as friends, and seek online one-to-one consulting services."));
    pTextLabel->setWordWrap(true);
    pTextLabel->setFont(ft);
    pTextLabel->setPalette(color_Introduction);

    // 初始化下拉按钮
    QHBoxLayout* m_hlayout =new QHBoxLayout;
    m_SystemComboBox = new DComboBox;
    m_SystemComboBox->setFont(ft);
    m_hlayout->addWidget(m_SystemComboBox);

    m_BlockComboBox = new DComboBox;
    m_BlockComboBox->setFont(ft);
    m_hlayout->addWidget(m_BlockComboBox);

    m_ItemComboBox = new DComboBox;
    m_ItemComboBox->setFont(ft);
    m_hlayout->addWidget(m_ItemComboBox);

    QVBoxLayout* pArchLayout = new QVBoxLayout;
    pArchLayout->addWidget(pTextLabel);
    pArchLayout->addLayout(m_hlayout);
    pArchFrame->setLayout(pArchLayout);

    return pArchFrame;
}

//初始化combo列表值
void ContactUs::initComboBox()
{
    //初始化system combobox
    m_SystemComboBox->addItems(devicetree::getInstance()->getSystemNames());
    //设置system combox 默认值为第一项，联动出发信号改变
    m_SystemComboBox->setCurrentIndex(0);
}

void ContactUs::slotUpdateComboBox()
{
    m_SystemComboBox->clear();
    m_SystemComboBox->addItems(devicetree::getInstance()->getSystemNames());
    m_SystemComboBox->setCurrentIndex(0);
}

//服务端返回客户信息
void ContactUs::slotUpdateCustomInfo(QByteArray data)
{
//根据服务端返回数据更新Card
//searchcustom :{   //
//name：””,
//team：””,
//tele：””,
//email：””,
//QRCode：””
//},
    QJsonParseError json_error;
    QJsonDocument jsonDoc(QJsonDocument::fromJson(data, &json_error));

    QJsonObject jsonObject = jsonDoc.object();

    if(jsonObject.isEmpty())
        return;

    ServiceCustomer customer;

    QString name = jsonObject["name"].toString();
    QString team = jsonObject["team"].toString();
    QString tele = jsonObject["moblie"].toString();
    QString email = jsonObject["email"].toString();
    QString QRCode = jsonObject["qrcode"].toString();

    if(QRCode != "")
    {
        QString qcodefile = devicetree::getInstance()->downQRCode(QRCode);

        QIcon icon(qcodefile);

        QImage image;

        image.load(qcodefile);

        QImage image1 = image.scaled(m_pQRCodeLabel->size(),Qt::IgnoreAspectRatio);//重新调整图像大小以适应label

        m_pQRCodeLabel->setPixmap(QPixmap::fromImage(image1));

    }

    m_pTelTextLabel->setText(tele);

    m_pEmailTextLabel->setText(email);

    devicetree::getInstance()->insertCustomerInfo(m_curSerchKey, customer);
}

//system combox 变化槽函数
void ContactUs::onSystemComboBoxIndexChanged(QString value)
{
    QStringList list  = devicetree::getInstance()->getBlockNames(value);

    //联动改变block combox 内容
    m_BlockComboBox->clear();
    m_BlockComboBox->addItems(list);
    m_BlockComboBox->setCurrentIndex(0);
}


//block combox 变化槽函数
void ContactUs::onBlockComboBoxIndexChanged(QString value)
{
    QStringList list  = devicetree::getInstance()->getItemNames(value);
    //联动改变最后面的槽函数
    m_ItemComboBox->clear();
    m_ItemComboBox->addItems(list);
    m_ItemComboBox->setCurrentIndex(0);
}

void ContactUs::onItemComboBoxIndexChanged(QString value)
{
    //修改完combobox，发送消息到服务器读取企业客服信息
    QString system = m_SystemComboBox->currentText();
    QString block = m_BlockComboBox->currentText();

    searchCustomerInfo(system, block, value);
}

//请求查找客服信息
void ContactUs::searchCustomerInfo(QString system, QString block, QString item)
{
    QJsonObject rootObj;

    rootObj.insert("system",system);
    rootObj.insert("block",block);
    rootObj.insert("item",item);

    m_curSerchKey = system + "-" + block + "-" + item;

    emit sigsearchCustomInfo(QJsonDocument(rootObj).toJson().simplified());
}


