#include "customerchatwidget.h"

#include <QHBoxLayout>
#include <QVBoxLayout>
#include <QLineEdit>
#include <QPushButton>
#include <QDebug>
#include <QKeyEvent>
#include <QFileDialog>
#include <QScrollBar>
#include <QUuid>
#include <QJsonParseError>
#include <QJsonDocument>
#include <QJsonObject>
#include <QJsonArray>
#include <QEventLoop>
#include <QStandardPaths>
#include <QFileIconProvider>
#include <DIconButton>
#include <DPalette>
#include <QApplication>
#include <QDesktopWidget>
#include <QDesktopServices>
#include <DFileDialog>
#include <QFileInfo>
#include <DMenu>
#include <DCommandLinkButton>
#include <QDBusInterface>
#include <QDBusReply>
#include <QTimer>
#include <QMimeData>
#include <DToolTip>
#include <QToolTip>
#include <DMessageManager>

#include "dsysinfo.h"
#include "../../clientsocket.h"

DCORE_USE_NAMESPACE

bool sortDate(QString dateone, QString datetwo)
{
    if(QDate::fromString(dateone,"yyyy.MM.dd") > QDate::fromString(datetwo, "yyyy.MM.dd"))
    {
        return true;
    }
    return false;
}

TitleFrame::TitleFrame(QWidget* parent ):DFrame(parent),m_sCustomerIcon(":/images/Avatar.svg")
{
    QHBoxLayout* phlayout = new QHBoxLayout;

    m_picon = new DLabel(this);
    m_picon->setPixmap(QIcon(m_sCustomerIcon).pixmap(35,35));

    QFont ft;
    ft.setPointSize(9);

    m_plabel = new DLabel;

    m_plabel->setFont(ft);

    phlayout->addWidget(m_picon);

    phlayout->addWidget(m_plabel);

    phlayout->addStretch(10);

    setLayout(phlayout);

    setFixedHeight(51);
}

TitleFrame::~TitleFrame()
{
}

void TitleFrame::setTitle(QString name, QString url)
{
    if(!url.isEmpty())
    {
        m_picon->setPixmap(QIcon(url).pixmap(35,35));
    }


    m_plabel->setText(name);
}



MessageFrame::MessageFrame(Message msg, QWidget* parent ) :
    DFrame(parent),
    m_bSendSuccess(false),
    m_pMessageEdit(nullptr),
    m_pmsgFrame(nullptr),m_pmsglabel(nullptr)
{
    m_pMainLayout = new QHBoxLayout;

    m_pMenu = new DMenu;

    m_pCurMessage = msg;

    QAction* pOpenAc = new QAction(tr("open file"));
    QAction* pSaveAsAc = new QAction(tr("save as"));

    m_pMenu->addAction(pOpenAc);
    m_pMenu->addAction(pSaveAsAc);

    connect(pOpenAc, &QAction::triggered, this, &MessageFrame::onOpenFile);
    connect(pSaveAsAc, &QAction::triggered, this, &MessageFrame::onSaveAsFile);

    initUI();

    setLayout(m_pMainLayout);

    m_pMainLayout->setContentsMargins(0,0,0,0);

    setFrameShape(QFrame::NoFrame);

    this->setSizePolicy(QSizePolicy::Preferred, QSizePolicy::Preferred);
}

MessageFrame::~MessageFrame()
{
}

QString MessageFrame::getMessageId()
{
    return m_pCurMessage.message_id;
}

void MessageFrame::setMessageRecive()
{
    m_bSendSuccess = true;

    if(m_pCurMessage.direct == "Receive")
        return;

    QIcon icon("://images/read.svg");

    m_pMessageSuccessLabel->setPixmap(icon.pixmap(15, 15));

    QTimer::singleShot(3000, this,  [=](){
        m_pMessageSuccessLabel->setVisible(false);
    });
}

Message MessageFrame::getMessage()
{
    return m_pCurMessage;
}

DFrame* MessageFrame::createFileFrame()
{
    DFrame* pWidget = new DFrame;

    QHBoxLayout* playot = new QHBoxLayout;
    QVBoxLayout* pvlayout = new QVBoxLayout;

    QFileInfo file_info(m_pCurMessage.content);
    QFileIconProvider icon_provider;
    QIcon icon = icon_provider.icon(file_info);

    DLabel* pnamelabel = new DLabel;
    DLabel* psizelabel = new DLabel;

    DLabel  *pIconLabel = new DLabel;
    pIconLabel->setFixedSize(50,50);

    pIconLabel->setPixmap(icon.pixmap(50,50));
    pIconLabel->setScaledContents(true);

    pnamelabel->setText(m_pCurMessage.filename);

    int filesize = m_pCurMessage.filesize.toInt();

    QString sz;
    QString unit = "B";
    if(filesize > 1024) {
            filesize /= 1024;
            unit = "Kb";
            if(filesize > 1024) {
            filesize /= 1024;
            unit = "Mb";
            if(filesize > 1024) {
            filesize /= 1024;
            unit = "Gb";
            }
        }
    }
    sz.append(QString::number(filesize)).append(unit);


    psizelabel->setText(sz);

    pvlayout->addWidget(pnamelabel);

    pvlayout->addWidget(psizelabel);

    pvlayout->setSpacing(3);

    playot->addLayout(pvlayout);

    playot->addWidget(pIconLabel);

    playot->setSpacing(20);

    playot->setContentsMargins(11,11,11,11);

    pWidget->setLayout(playot);

    pWidget->setBackgroundRole(DPalette::ItemBackground);

    pWidget->setFixedSize(250, 88);

    pWidget->setContentsMargins(3,3,3,3);

    pWidget->installEventFilter(this);

    pWidget->setContextMenuPolicy(Qt::CustomContextMenu);

    connect(pWidget, &QLabel::customContextMenuRequested, [=](const QPoint pos){
        m_pMenu->exec(QCursor::pos());
    });

    return pWidget;
}


DLabel* MessageFrame::createImageLabel()
{
    DLabel* label = new DLabel(this);

    QPixmap pixmap(m_pCurMessage.content);

    int width = 100;
    int height = 100;

    QDesktopWidget* desktop = QApplication::desktop();

    int screenwidth = desktop->availableGeometry().width();
    int screenheigth = desktop->availableGeometry().width();

    int pixwidth = pixmap.width();
    int pixheight = pixmap.height();

    if(pixwidth >= screenwidth || pixheight >= screenheigth)
    {
        width = 360;
        height = width* (pixheight / (pixwidth*1.0));
    }
    else
    {
        float f1 = (pixwidth / (screenwidth * 1.0));
        float f2 = (pixheight / (screenheigth * 1.0));

        width = 500 * f1;
        height = 500 * f2;
    }

    label->setPixmap(pixmap.scaled(width, height));

    label->installEventFilter(this);

    label->setContextMenuPolicy(Qt::CustomContextMenu);

    connect(label, &QLabel::customContextMenuRequested, [=](const QPoint pos){
        m_pMenu->exec(QCursor::pos());
    });

    return label;
}


void MessageFrame::onSaveAsFile()
{
    QFileInfo info(m_pCurMessage.content);

    QString filename =
            DFileDialog::getSaveFileName(this, tr("Save File"),
                                         "/home",
                                         "file (*."+ info.completeSuffix() +")");

    if(filename.isEmpty())
    {
        return;
    }

    QFile::copy(m_pCurMessage.content, filename);
}

void MessageFrame::onOpenFile()
{
    QDesktopServices::openUrl(QUrl::fromLocalFile(m_pCurMessage.content));
}

void MessageFrame::initUI()
{

    QFont ft;
    ft.setPointSize(9);

    if(m_pCurMessage.direct == "Receive")
    {
        //创建消息控件
        QHBoxLayout* phlayout = new QHBoxLayout;

        QVBoxLayout* play = new QVBoxLayout;

        m_pTimeLabel = new DLabel;
        m_pTimeLabel->setFont(ft);

        m_pIconLabel = new DLabel;

        m_pNameLabel = new DLabel(m_pCurMessage.name);
        m_pNameLabel->setFont(ft);

        phlayout->setSpacing(10);
        phlayout->addStrut(10);
        phlayout->addWidget(m_pIconLabel);
        phlayout->addWidget(m_pNameLabel);
        phlayout->addWidget(m_pTimeLabel);

        m_pIconLabel->setFixedSize(35,35);

        QIcon icon(m_pCurMessage.icon);
        m_pIconLabel->setPixmap(icon.pixmap(35,35));
        m_pIconLabel->setScaledContents(true);

        QDateTime datetime = QDateTime::fromString(m_pCurMessage.datetime, "yyyy.MM.dd hh:mm:ss");
        QString time = datetime.toLocalTime().toString("H:mm AP");
        m_pTimeLabel->setText(time);

        phlayout->addStretch(5);
        phlayout->setSpacing(10);
        play->addLayout(phlayout);

        if(m_pCurMessage.type == "Message")
        {
            m_pMessageEdit = new QTextEdit(this);

            m_pMessageEdit->setFont(ft);

            connect(m_pMessageEdit->document(),&QTextDocument::contentsChange,this,&MessageFrame::adjustContent);

            m_pMessageEdit->setMaximumWidth(300);

            m_pMessageEdit->setText(m_pCurMessage.content);

            m_pMessageEdit->setTextColor(Qt::white);

            m_pMessageEdit->setReadOnly(true);

            //m_pMessageEdit->setSizeAdjustPolicy(QAbstractScrollArea::AdjustToContents);

            m_pMessageEdit->setVerticalScrollBarPolicy(Qt::ScrollBarAlwaysOff);

            m_pMessageEdit->setHorizontalScrollBarPolicy(Qt::ScrollBarAlwaysOff);

            m_pMessageEdit->setStyleSheet("background-color:rgba(0,0,0,0.03); border-radius:5px;");

            m_pMessageEdit->setContentsMargins(8, 8, 8, 8);


           m_pMessageEdit->setMouseTracking(true);

           m_pMessageEdit->viewport()->setMouseTracking(true);
		   
            m_pMessageEdit->viewport()->installEventFilter(this);

            QHBoxLayout* pl = new QHBoxLayout;

            pl->addWidget(m_pMessageEdit);

            pl->addStretch(10);

            play->addLayout(pl);
        }
        else if(m_pCurMessage.type == "Image")
        {
            m_pmsglabel = createImageLabel();

            play->addWidget(m_pmsglabel);

        }
        else if(m_pCurMessage.type == "File")
        {
            //文件对话框
            m_pmsgFrame = createFileFrame();

            play->addWidget(m_pmsgFrame);
        }

        play->setSpacing(10);

        m_pMainLayout->addLayout(play);
        m_pMainLayout->addStretch(5);

    }
    else if(m_pCurMessage.direct == "Send")
    {
        QVBoxLayout* play = new QVBoxLayout;
        //创建消息控件
        QHBoxLayout* phlayout = new QHBoxLayout;

        m_pIconLabel = new DLabel;

        m_pTimeLabel = new DLabel;

        m_pNameLabel = new DLabel(m_pCurMessage.name);
        m_pTimeLabel->setFont(ft);
        m_pNameLabel->setFont(ft);

        QDateTime datetime = QDateTime::fromString(m_pCurMessage.datetime, "yyyy.MM.dd hh:mm:ss");

        QString time = datetime.toLocalTime().toString("H:mm AP");

        m_pTimeLabel->setText(time);

        phlayout->addStretch(10);
        phlayout->addWidget(m_pTimeLabel);
        phlayout->addWidget(m_pNameLabel);
        phlayout->addWidget(m_pIconLabel);
        phlayout->setSpacing(10);


        m_pIconLabel->setFixedSize(35,35);

        m_pIconLabel->setPixmap(QIcon(m_pCurMessage.icon).pixmap(35,35));

        m_pIconLabel->setScaledContents(true);

        play->addLayout(phlayout);

        if(m_pCurMessage.type == "Message")
        {
            //rgba 0,129,255,1
           m_pMessageEdit = new QTextEdit(this);

           m_pMessageEdit->setFont(ft);

           connect(m_pMessageEdit->document(),&QTextDocument::contentsChange,this,&MessageFrame::adjustContent);

           m_pMessageEdit->setMaximumWidth(300);

           m_pMessageEdit->setText(m_pCurMessage.content);

           m_pMessageEdit->setTextColor(Qt::white);

           m_pMessageEdit->setReadOnly(true);

           m_pMessageEdit->setVerticalScrollBarPolicy(Qt::ScrollBarAlwaysOff);

           m_pMessageEdit->setHorizontalScrollBarPolicy(Qt::ScrollBarAlwaysOff);

           m_pMessageEdit->setStyleSheet("background-color:rgba(0,129,255,1); color:white;border-radius:5px;");

           m_pMessageEdit->viewport()->setContentsMargins(20,0,20,0);

           m_pMessageEdit->setMouseTracking(true);

           m_pMessageEdit->viewport()->setMouseTracking(true);

           m_pMessageEdit->viewport()->installEventFilter(this);

           m_pMessageEdit->setAlignment(Qt::AlignLeft);

           QHBoxLayout* pl = new QHBoxLayout;

           pl->addStretch(10);

           pl->addWidget(m_pMessageEdit);

           play->addLayout(pl);


        }
        else if(m_pCurMessage.type == "Image")
        {
            m_pmsglabel = createImageLabel();

            play->addWidget(m_pmsglabel);
        }
        else if(m_pCurMessage.type == "File")
        {
            //文件对话框
            m_pmsgFrame = createFileFrame();

            play->addWidget(m_pmsgFrame);
        }

        play->setSpacing(10);

        m_pMainLayout->addStretch(5);

        //消息发送成功失败标志
        m_pMessageSuccessLabel = new DLabel();

        m_pMessageSuccessLabel->setAlignment(Qt::AlignBottom);

        QIcon icon("://images/unread.svg");

        m_pMessageSuccessLabel->setPixmap(icon.pixmap(15,15));

        m_pMainLayout->addWidget(m_pMessageSuccessLabel);

        m_pMainLayout->addLayout(play);
    }
}



void MessageFrame::adjustContent()
{
    QTextDocument *document=qobject_cast<QTextDocument*>(sender());

    document->adjustSize();

    if(document)
    {

        document->setDocumentMargin(8);

        QTextEdit *editor = qobject_cast<QTextEdit*>(document->parent()->parent());

        if(editor)
        {

            QString text = editor->toPlainText().trimmed();

            if(text.isEmpty())
            {
               return;
            }

            QFontMetrics mert  = editor->viewport()->fontMetrics();
            int nCount = text.count("\n");


            int width = mert.width(text);
            int height = mert.height();

            if(width > editor->maximumWidth())
            {

                float fheight = (width / (300*1.0)) + nCount;
                int nheight = (width / (300*1.0)) + nCount;
                float d = fheight - nheight;
                if(fheight - nheight > 0 )
                {
                    nheight ++;
                }
                if(d < 0.5)
                {
                    editor->setFixedSize(300, nheight * height + 28 );
                }
                else {
                    editor->setFixedSize(300, nheight * height + 30 );
                }



            }
            else
            {
                editor->setFixedSize(width + 28, 36);
            }
        }

    }
}

bool MessageFrame::eventFilter(QObject *o, QEvent *e)
{
    if(o == m_pmsgFrame || o == m_pmsglabel)
    {
         if(e->type() == QEvent::MouseButtonDblClick)
         {
             if(m_pCurMessage.type == "Image" || m_pCurMessage.type == "File")
             {
                 QDesktopServices::openUrl(QUrl::fromLocalFile(m_pCurMessage.content));
             }
         }
    }

    if(m_pMessageEdit != nullptr)
    {
        if(o == m_pMessageEdit->viewport())
        {
            if(e->type() == QEvent::Wheel)
            {
                return true;
            }
        }
    }

    return QFrame::eventFilter(o, e);
}

ChatFrame::ChatFrame(QWidget* parent) : DScrollArea(parent),
    m_pServerName(""),
    m_pServerIcon("://images/Avatar.svg"),
    m_pCustomIcon("://images/user.png"),
    m_pCustomName("me"),
    m_curHistoryIndex(0)
{
    QWidget* pWidget = new QWidget(this);

    m_pLayout = new QVBoxLayout;

    m_pLayout->setSpacing(10);

    m_pLayout->setContentsMargins(20,20,20,20);

    m_pLayout->addStretch(10);

    pWidget->setLayout(m_pLayout);

    this->setWidgetResizable(true);

    this->setVerticalScrollBarPolicy(Qt::ScrollBarAsNeeded);

    m_pVerBar = new QScrollBar;

    this->setVerticalScrollBar(m_pVerBar);

    this->setSizeAdjustPolicy(QAbstractScrollArea::AdjustToContents);

    this->setWidget(pWidget);
}

ChatFrame::~ChatFrame()
{
}

void ChatFrame::setCustomTitle(QString name,QString icon)
{
    m_pCustomIcon = icon;
    m_pCustomName = name;
}

void ChatFrame::setServiceTitle(QString name,QString icon)
{
    m_pServerName = name;
    if(!icon.isEmpty())
    {
          m_pServerIcon = icon;
    }

}

void ChatFrame::setHistoryBtn(DCommandLinkButton* pbtn)
{
    m_pHistoryBtn = pbtn;
}

void ChatFrame::createMessageLog()
{
    QString oldpath = QDir::currentPath();

    QString savepath = QStandardPaths::writableLocation(QStandardPaths::AppLocalDataLocation) + "/log/";

    QDir* m_pLogDir = new QDir;

    if(!m_pLogDir->exists(savepath))
    {
        m_pLogDir->mkpath(savepath);
    }

    //将程序的执行路径设置到filePath下
    m_pLogDir->setCurrent(savepath);

    //初始化历史消息文件列表
    QFileInfoList fileInfoList = m_pLogDir->entryInfoList();

    int totalFileSize = 0;
    foreach (QFileInfo fileInfo, fileInfoList)
    {
        QString fileext = fileInfo.suffix();

        if(fileext == "log")
        {

            totalFileSize += fileInfo.size();

            if(fileInfo.size() != 0)
            {
                m_pHistorys.push_back(fileInfo.completeBaseName());
            }
        }
    }

    if(totalFileSize == 0)
    {
        m_pHistoryBtn->setEnabled(false);
    }
    else {
        m_pHistoryBtn->setEnabled(true);
    }

    qSort(m_pHistorys.begin(), m_pHistorys.end(), sortDate);

    m_pLogDir->setPath(oldpath);

    return;
}


void ChatFrame::saveMessage(Message msg)
{   
    QJsonObject rootObj;

    rootObj.insert("datetime", msg.datetime);

    rootObj.insert("type", msg.type);

    if(msg.direct == "Send")
    {
        rootObj.insert("name",m_pCustomName);
        rootObj.insert("icon",m_pCustomIcon);
        msg.name = m_pCustomName;
        msg.icon = m_pCustomIcon;
    }
    else
    {
        rootObj.insert("name",m_pServerName);
        rootObj.insert("icon",m_pServerIcon);
        msg.name = m_pServerName;
        msg.icon = m_pServerIcon;
    }
    rootObj.insert("direct",msg.direct);
    rootObj.insert("message_id", msg.message_id);
    rootObj.insert("content", msg.content);
    rootObj.insert("filename",msg.filename);
    rootObj.insert("filesize",  msg.filesize);

    QByteArray data = QJsonDocument(rootObj).toJson().simplified();
    data.push_back("\n");

    writeLogFile(data);
}

void ChatFrame::writeLogFile(QByteArray data)
{
    QString oldpath = QDir::currentPath();

    QString savename = QDate::currentDate().toString("yyyy.MM.dd")+".log";

    QString savepath = QStandardPaths::writableLocation(QStandardPaths::AppLocalDataLocation) + "/log/";

    QDir* m_pLogDir = new QDir;

    if(!m_pLogDir->exists(savepath))
    {
        m_pLogDir->mkpath(savepath);
    }

    //将程序的执行路径设置到filePath下
    m_pLogDir->setCurrent(savepath);

    QFile historyfile;

    historyfile.setFileName(savename);

    if(!historyfile.open(QIODevice::WriteOnly | QIODevice::Text| QIODevice::Append))
    {
        qDebug()<<QObject::tr("open failed");
    }

    historyfile.write(data);

    historyfile.close();

    m_pLogDir->setPath(oldpath);
}

//读取历史消息，每次读取一个日期的历史消息
QJsonArray ChatFrame::readMessages(QString name)
{
    QFile readFile(name);

    QJsonArray histroyAry;

    if(readFile.open(QFile::ReadOnly))
    {
        while(!readFile.atEnd())
        {
            QByteArray line = readFile.readLine();

            QJsonParseError json_error;
            QJsonDocument jsonDoc(QJsonDocument::fromJson(line.simplified(), &json_error));

            if(json_error.error != QJsonParseError::NoError)
            {
                qDebug() << "json error!";
            }

            QDateTime linetime = QDateTime::fromString(jsonDoc.object()["datetime"].toString(),"yyyy.MM.dd hh:mm:ss");
            QDateTime earlytime = QDateTime::fromString(m_CurEarlyDateTime,"yyyy.MM.dd hh:mm:ss");

            if(linetime < earlytime)
            {
                histroyAry.push_back(jsonDoc.object());
            }
        }
    }

    return histroyAry;
}


void ChatFrame::onReadHistroy()
{
    QString oldpath = QDir::currentPath();

    QString savepath = QStandardPaths::writableLocation(QStandardPaths::AppLocalDataLocation) + "/log/";

    QDir::setCurrent(savepath);

    int maxindex = m_pHistorys.size()-1;

    if(m_curHistoryIndex <= maxindex)
    {
        QString date = m_pHistorys.at(m_curHistoryIndex);

        QJsonArray msgs = readMessages( date + ".log");

        //如果消息为空，则需要读取下一个日志文件消息．
        if(msgs.isEmpty())
        {
            return;
        }

        //历史消息合法判断

        //插入历史消息
        insertHistroyMessages(msgs);

        //插入历史消息所对应的当日日期
        DLabel* pDateLabel = new DLabel;

        pDateLabel->setAlignment(Qt::AlignCenter);
        pDateLabel->setText(date);

        m_pLayout->insertWidget(0, pDateLabel);

        m_curHistoryIndex++;

        if(m_curHistoryIndex > maxindex)
        {
            m_pHistoryBtn->setEnabled(false);
        }
    }
    else
    {
        m_pHistoryBtn->setEnabled(false);
    }

    QDir::setCurrent(oldpath);
}

//在当前控件前插入历史消息
void ChatFrame::insertHistroyMessages(QJsonArray messages)
{
    int nsize = messages.count()-1;

    for(int i = nsize; i >= 0; i--)
    {
        QJsonValue value = messages.takeAt(i);
        QJsonObject object = value.toObject();
        Message msg;

        msg.type = object["type"].toString();
        msg.message_id = object["message_id"].toString();
        msg.direct = object["direct"].toString();
        msg.content = object["content"].toString();
        msg.filename = object["filename"].toString();
        msg.filesize = object["filesize"].toString();
        msg.name = object["name"].toString();
        msg.icon = object["icon"].toString();
        msg.datetime = object["datetime"].toString();

        MessageFrame* pFrame = new MessageFrame(msg, this->widget());
        pFrame->setMessageRecive();
        m_pLayout->insertWidget(0, pFrame);
    }
}

//添加状态消息,显示绘话状态字符串
void ChatFrame::addStatusMessage(QString value)
{
   QFont ft;
   ft.setPointSize(8);
   QHBoxLayout* p1 = new QHBoxLayout;
   QHBoxLayout* p2 = new QHBoxLayout;

   DLabel* pTimeLabel = new DLabel(this);
   DLabel* pContentLabel = new DLabel(this);
   pTimeLabel->setFont(ft);
   pContentLabel->setFont(ft);

   p1->addWidget(pTimeLabel);
   p2->addWidget(pContentLabel);

   QString msgtime = QDateTime::currentDateTime().toString("yyyy.MM.dd hh:mm:ss");
   pTimeLabel->setText(msgtime);
   pContentLabel->setText(value);

   QVBoxLayout* pLayout = new QVBoxLayout;

   pLayout->addLayout(p1);
   pLayout->addLayout(p2);

   p1->setAlignment(Qt::AlignHCenter);
   p2->setAlignment(Qt::AlignHCenter);

   p1->setContentsMargins(0,0,0,0);
   p2->setContentsMargins(0,0,0,0);

   pLayout->setSpacing(0);
   pLayout->setAlignment(Qt::AlignHCenter);
   pLayout->setContentsMargins(0,0,0,0);

   int count = m_pLayout->count();

   m_pLayout->insertLayout(count-1, pLayout);

   m_pVerBar->setRange(0, this->widget()->height());

   m_pVerBar->setValue(m_pVerBar->maximum());
}

//聊天框中添加消息，每次添加在最后面
void ChatFrame::addMessage(Message msg)
{
    if(msg.direct == "Send")
    {
        msg.icon = m_pCustomIcon;
        msg.name = m_pCustomName;
    }
    else if(msg.direct == "Receive")
    {
        msg.icon = m_pServerIcon;
        msg.name = m_pServerName;
    }

    msg.datetime = QDateTime::currentDateTime().toString("yyyy.MM.dd hh:mm:ss");

    MessageFrame* pFrame = new MessageFrame(msg, this->widget());

    int count = m_pLayout->count();

    m_pLayout->insertWidget(count-1, pFrame);

    m_pVerBar->setRange(0, this->widget()->height());

    //所有消息列表
    m_pMessages.push_back(pFrame);

    m_pVerBar->setValue(m_pVerBar->maximum());

    //接收到的消息，直接保存。不需要处理传输成功状态
    if(msg.direct == "Receive")
    {
        saveMessage(msg);
    }

}

void ChatFrame::setCurEarlyDateTime(QString str)
{
    m_CurEarlyDateTime = str;
}

void ChatFrame::setMessageBack(QString msgid)
{
    QList<MessageFrame*>::iterator it;

    for(it = m_pMessages.begin(); it != m_pMessages.end(); it++)
    {
        MessageFrame* curMsgFrame = *it;

        if(curMsgFrame->getMessageId() == msgid)
        {
            curMsgFrame->setMessageRecive();

            //发送成功消息保存
            Message msg = curMsgFrame->getMessage();

            //发送消息成功，保存当前消息
            saveMessage(msg);

            break;
        }
    }
}

CustomerChatWidget::CustomerChatWidget(QWidget *parent) :
    DFrame(parent),
    m_customer_token(""),
    m_session_id(0),
    m_user_uuid(""),
    m_custom_session_code(0)
{
    //初始化界面布局
    initUI();

    getSystemIconName();

    this->installEventFilter(this);
}

CustomerChatWidget::~CustomerChatWidget()
{
    closeSession();
}

void CustomerChatWidget::getSystemIconName()
{

    QDBusInterface manual("com.deepin.daemon.Accounts",
                             "/com/deepin/daemon/Accounts/User1000",
                             "com.deepin.daemon.Accounts.User",
                             QDBusConnection::systemBus());

    m_icon = manual.property("IconFile").toString();

    if(m_icon.startsWith("file"))
        m_icon = m_icon.mid(7);

    m_name = manual.property("UserName").toString();

    m_user_uuid = manual.property("UUID").toString();

    m_pChatWidget->setCustomTitle(m_name, m_icon);
}

//布局界面
void CustomerChatWidget::initUI()
{
    QFont ft;
    ft.setPointSize(9);
    QVBoxLayout* pvlayout = new QVBoxLayout;

    DHorizontalLine* hline = new DHorizontalLine;
    DHorizontalLine* hline1 = new DHorizontalLine;
    DHorizontalLine* hline2 = new DHorizontalLine;

    m_pTitleWidget = new TitleFrame(this);
    m_pTitleWidget->setFrameRounded(false);
    m_pTitleWidget->setFrameShape(QFrame::NoFrame);

    QHBoxLayout* phistorylayout = new QHBoxLayout;

    m_pHistoryBtn = new DCommandLinkButton(tr("View history"), this);
    m_pHistoryBtn->setFont(ft);

    phistorylayout->addWidget(m_pHistoryBtn);

    phistorylayout->setAlignment(Qt::AlignHCenter);

    m_pChatWidget = new ChatFrame(this);
    m_pChatWidget->setFrameShape(QFrame::NoFrame);

    m_pChatWidget->setHistoryBtn(m_pHistoryBtn); //首先设置历史按键

    m_pChatWidget->createMessageLog();    //其次读取创建历史消息文件

    connect(m_pHistoryBtn, &DCommandLinkButton::clicked, m_pChatWidget, &ChatFrame::onReadHistroy);

    DFrame* pBtnFrame = new DFrame(this);

    pBtnFrame->setFrameRounded(false);
    pBtnFrame->setFrameShape(QFrame::NoFrame);
    m_pOpenFileButton = new DIconButton(this);
    m_pOpenFileButton->setIcon(QIcon(":/images/file.svg"));
    m_pOpenFileButton->setIconSize(QSize(36,36));
    m_pOpenFileButton->setFlat(true);
    m_pOpenFileButton->setFixedSize(40,40);

    QHBoxLayout* phlayout = new QHBoxLayout;
    phlayout->addWidget(m_pOpenFileButton);
    phlayout->addStretch(5);
    pBtnFrame->setLayout(phlayout);
    phlayout->setContentsMargins(8,3,0,3);
    pBtnFrame->setContentsMargins(0,0,0,0);
    pBtnFrame->setFixedSize(100, 45);

    m_pUserInputWidget = new QTextEdit(this);

    m_pEnterTipsLabel = new DLabel(tr("Press ⏎ Enter to send messages"));
    m_pEnterTipsLabel->setFont(ft);

    m_pUserInputWidget->setFrameShape(QFrame::NoFrame);

    connect(m_pOpenFileButton, &DIconButton::clicked, this, &CustomerChatWidget::slotOpenFile);

    m_pUserInputWidget->setPlaceholderText(tr("Welcome to contact us, please describe your questions"));
    m_pUserInputWidget->setFont(ft);

    m_pUserInputWidget->setFixedHeight(116);
    pvlayout->addWidget(m_pTitleWidget);
    pvlayout->addWidget(hline);
    pvlayout->addLayout(phistorylayout);
    pvlayout->addWidget(m_pChatWidget);
    pvlayout->addWidget(hline1);
    pvlayout->addWidget(pBtnFrame);
    pvlayout->addWidget(hline2);
    pvlayout->addWidget(m_pUserInputWidget);
    pvlayout->addWidget(m_pEnterTipsLabel,0,Qt::AlignRight);

    pvlayout->setContentsMargins(0, 0, 15, 10);
    pvlayout->setSpacing(0);

    setLayout(pvlayout);

    m_pUserInputWidget->setAcceptDrops(true);
    m_pUserInputWidget->setMouseTracking(true);

    m_pUserInputWidget->setObjectName("lineedit");

    m_pUserInputWidget->installEventFilter(this);
    m_pUserInputWidget->viewport()->installEventFilter(this);
}


bool CustomerChatWidget::eventFilter(QObject *o, QEvent *e)
{
    if(o == m_pUserInputWidget)
    {
         if(e->type() == QEvent::KeyPress)
         {
             QKeyEvent* peve = dynamic_cast<QKeyEvent*>(e);
             if(peve->key() == Qt::Key_Return || peve->key() == Qt::Key_Enter)
             {
                 sendMessage();
                 return true;
             }
         }
    }
    else if(o == this)
    {
        if(e->type() == QEvent::Show)
        {
            if(!m_session_id)
            {
                createSession();
            }
        }
    }
    else if(o == m_pUserInputWidget->viewport())
    {
        if(e->type() == QEvent::DragEnter)
        {
            QDragEnterEvent* eve = dynamic_cast<QDragEnterEvent*>(e);

            if(eve->mimeData()->hasFormat("text/uri-list"))
            {
              QList<QUrl> urls =  eve->mimeData()->urls();

              foreach(QUrl url, urls)
              {
                  QFileInfo fi(url.toLocalFile());
                  if(fi.isDir())//屏蔽文件夹，不支持拖拽文件夹
                  {
                      eve->ignore();
                      return true;
                  }
              }
              eve->acceptProposedAction();
            }
            else
            {
               eve->ignore();
            }
        }
        if(e->type() == QEvent::Drop)
        {
            QDropEvent* eve = dynamic_cast<QDropEvent*>(e);
            const QMimeData *mineData = eve->mimeData();//获取MIME数据

            QList<QUrl>urlList=mineData->urls();    //获取URL列表

            if(urlList.isEmpty())
                return true;
            //将其中第一个URL表示为本地文件路径
            QString fileName=urlList.at(0).toLocalFile();
            if(fileName.isEmpty())
                return true;

            if(!fileName.isEmpty())                 //若文件路径不为空
            {
               sendFile(fileName);
            }
            return true;
        }

    }

    return QFrame::eventFilter(o, e);
}


void CustomerChatWidget::sendFile(QString url)
{
    QFileInfo info(url);

    Message msg;
    msg.message_id = QUuid::createUuid().toString();
    msg.direct = "Send";
    msg.content = info.filePath();
    QString type;
    QString ext = info.completeSuffix();

    if(ext == "png"  ||ext == "jpg" ||ext == "jpeg")
    {
      msg.type = "Image";
    }
    else
    {
      msg.type = "File";
    }

    msg.filename = info.fileName();
    msg.filesize = QString("%1").arg(info.size());

    if(! DNetState::getInstance()->getNetState())
    {
        QFont ft;
        ft.setPointSize(9);
        DFloatingMessage *fmsg = new DFloatingMessage();
        fmsg->setFont(ft);
        fmsg->setDuration(800);
        fmsg->setMessage(tr("network unavailable,message can not send Successful"));

        DMessageManager::instance()->sendMessage(m_pChatWidget, fmsg);

        return;
    }

    //上传文件
    m_pChatWidget->addMessage(msg);


    if(m_custom_session_code == 2001)
    {
        QFont ft;
        ft.setPointSize(9);
        DFloatingMessage *fmsg = new DFloatingMessage();
        fmsg->setFont(ft);
        fmsg->setDuration(800);
        fmsg->setMessage( tr("Customer service is busy, waiting for information check"));

        DMessageManager::instance()->sendMessage(m_pChatWidget, fmsg);

        m_pMessageUnArrived.push_back(msg);
    }
    else if(m_custom_session_code == 2002)
    {
        QFont ft;
        ft.setPointSize(9);
        DFloatingMessage *fmsg = new DFloatingMessage();
        fmsg->setFont(ft);
        fmsg->setDuration(800);
        fmsg->setMessage(tr("Customer service is not online, message sending failed"));

        DMessageManager::instance()->sendMessage(m_pChatWidget, fmsg);
    }
    else
    {
        if(m_session_id == 0)
        {
            createSession();
            m_pMessageUnArrived.push_back(msg);
        }
        else
        {
            emit signalUploadFile(url, msg.message_id);
        }
    }
}


void CustomerChatWidget::sendMessage()
{
    QString msg = m_pUserInputWidget->toPlainText();

    // 把最后一个回车换行符删掉
    while (msg.endsWith("\n")) {
        msg.remove(msg.length() - 2, 2);
    }

    if(msg.trimmed().isEmpty())
    {

        QFont ft;
        ft.setPointSize(9);
        DFloatingMessage *msg = new DFloatingMessage();
        msg->setFont(ft);
        msg->setDuration(500);
        msg->setMessage( tr("Cannot send a blank message"));

        DMessageManager::instance()->sendMessage(m_pChatWidget, msg);

        return ;
    }


    if(msg.length() > 1000)
    {
        QFont ft;
        ft.setPointSize(9);
        DFloatingMessage *msg = new DFloatingMessage();
        msg->setFont(ft);
        msg->setDuration(800);
        msg->setMessage( tr("Cannot send message count > 1000"));

        DMessageManager::instance()->sendMessage(m_pChatWidget, msg);
        return ;
    }



    Message message;

    QJsonObject rootObj;
    QString msgid = QUuid::createUuid().toString().remove("{").remove("}");
    rootObj.insert("customer_token", m_customer_token);
    rootObj.insert("im_sub_session_id",m_session_id);
    rootObj.insert("message_id",  msgid);
    rootObj.insert("type","message");

    message.direct = "Send";
    message.type =  "Message";
    message.message_id = msgid;
    message.content = msg.trimmed();

    QJsonObject dataobject;

    dataobject.insert("content", msg.trimmed());

    rootObj.insert("data", QJsonValue(dataobject));

    // 清除输入框
    m_pUserInputWidget->clear();

    if(! DNetState::getInstance()->getNetState())
    {
        QFont ft;
        ft.setPointSize(9);
        DFloatingMessage *msg = new DFloatingMessage();
        msg->setFont(ft);
        msg->setDuration(800);
        msg->setMessage( tr("network unavailable,message can not send Successful"));

        DMessageManager::instance()->sendMessage(m_pChatWidget, msg);

        return;
    }

    if(m_custom_session_code == 2001)
    {
        QFont ft;
        ft.setPointSize(9);
        DFloatingMessage *msg = new DFloatingMessage();
        msg->setFont(ft);
        msg->setDuration(800);
        msg->setMessage( tr("Customer service is busy, waiting for information check"));

        DMessageManager::instance()->sendMessage(m_pChatWidget, msg);

        m_pMessageUnArrived.push_back(message);
    }
    else if(m_custom_session_code == 2002)
    {
        QFont ft;
        ft.setPointSize(9);
        DFloatingMessage *msg = new DFloatingMessage();
        msg->setFont(ft);
        msg->setDuration(800);
        msg->setMessage( tr("Customer service is not online, message sending failed"));

        DMessageManager::instance()->sendMessage(m_pChatWidget, msg);

    }
    else
    {
        if(m_session_id == 0)
        {
            createSession();
            m_pMessageUnArrived.push_back(message);
            return;
        }
        else
        {
            //发送消息
            emit signalMessage(QJsonDocument(rootObj).toJson().simplified());
        }

    }


    m_pChatWidget->addMessage(message);

}


void CustomerChatWidget::slotOpenFile()
{

    QString url = QFileDialog::getOpenFileName(this, tr("Open File"),
                                                      QStandardPaths::writableLocation(QStandardPaths::HomeLocation),
                                                      tr("Images (*)"));

    if(!url.isEmpty())
    {
       sendFile(url);
    }
}

void CustomerChatWidget::slotUploadFileSuccess(QString weburl, QByteArray  data)
{
    QJsonParseError json_error;
    QJsonDocument jsonDoc(QJsonDocument::fromJson(data.simplified(), &json_error));

    if(json_error.error != QJsonParseError::NoError)
    {
        qDebug() << "json error!";
    }

    QJsonObject jsonObject = jsonDoc.object();

    QString upload_path = jsonObject["fileurl"].toString();
    QString message_id = jsonObject["message_id"].toString();

    upload_path = weburl + "/" +upload_path;

    QFileInfo info(upload_path);

    double filesize = jsonObject["filesize"].toDouble();
    QString filename = jsonObject["filename"].toString();

    QString type ;
    if(filename.contains("jpg") || filename.contains("jpeg") || filename.contains("png"))
    {
        type = "image";
    }
    else
    {
        type = "file";
    }

    //组织文件发送 json结构
    QJsonObject rootObj;

    rootObj.insert("customer_token", m_customer_token);
    rootObj.insert("im_sub_session_id",m_session_id);
    rootObj.insert("message_id",  message_id);
    rootObj.insert("type",type);

    QJsonObject dataobject;

    dataobject.insert("content",  upload_path);
    dataobject.insert("filename", filename);

    QString sz;
    QString unit = "B";
    if(filesize > 1024) {
        filesize /= 1024;
        unit = "Kb";
        if(filesize > 1024) {
        filesize /= 1024;
        unit = "Mb";
        if(filesize > 1024) {
        filesize /= 1024;
        unit = "Gb";
        }
        }
    }
    sz.append(QString::number(filesize)).append(unit);

    dataobject.insert("filesize", sz);
    rootObj.insert("data", QJsonValue(dataobject));

    //发送文件消息
    emit signalMessage(QJsonDocument(rootObj).toJson().simplified());
}

QString CustomerChatWidget::getCustomSesson()
{
    QString uid = m_user_uuid.left(8);
    return "uos-" + m_name + "-"+ uid;
}

void CustomerChatWidget::createSession()
{
    //customertoken //每次会话创建
    m_customer_token = getCustomSesson();

    QJsonObject jsonObject;
    jsonObject.insert("customer_token", m_customer_token);

    emit sigStartSession(QJsonDocument(jsonObject).toJson().simplified());

    m_pChatWidget->setCurEarlyDateTime(QDateTime::currentDateTime().toString("yyyy.MM.dd hh:mm:ss"));
}

//客户主动关闭客服
void CustomerChatWidget::closeSession()
{
    Message message;

    QJsonObject rootObj;

    QString msgid = QUuid::createUuid().toString().remove("{").remove("}");

    rootObj.insert("customer_token", m_customer_token);
    rootObj.insert("im_sub_session_id",m_session_id);
    rootObj.insert("message_id",  msgid);
    rootObj.insert("type", "close");

    message.direct = "Send";
    message.type =  "close";
    message.message_id = msgid;
    message.content = "会话关闭";

    QJsonObject dataobject;

    dataobject.insert("content", message.content);

    rootObj.insert("data", QJsonValue(dataobject));

    //发送消息
    emit signalMessage(QJsonDocument(rootObj).toJson().simplified());
}

QString CustomerChatWidget::downServiceFile(QString urlpath)
{
    QString oldpath = QDir::currentPath();

    QFileInfo info(urlpath);

    QString savename = info.fileName();

    QString saveDir = QStandardPaths::writableLocation(QStandardPaths::CacheLocation);


    QDir CacheDir;

    if(!CacheDir.exists(saveDir))
    {
        CacheDir.mkpath(saveDir);
    }

    QDir::setCurrent(saveDir);


    QString savepath = saveDir + "/" + savename;

    //构造请求
    QNetworkAccessManager manager;
    QNetworkRequest request;
    request.setUrl(urlpath);
    // 发送请求
    QNetworkReply *reply = manager.get(request);
    //开启一个局部的事件循环，等待响应结束，退出
    QEventLoop loop;
    connect(reply, SIGNAL(finished()), &loop, SLOT(quit()));
    loop.exec();

    //判断是否出错,出错则结束
    if (reply->error() != QNetworkReply::NoError)
    {
        return "";
    }

    //保存文件
    QFile file(savename);
    if(!file.open(QIODevice::WriteOnly))
        return "";

    file.write(reply->readAll());

    file.close();

    delete reply;

    QDir::setCurrent(oldpath);

    return savepath;
}

//会话失败 2001 2002
void CustomerChatWidget::slotSessionFault(double code, QString errStr)
{

    //消息框显示状态显示值
    if(m_custom_session_code != code)
    {
        m_pChatWidget->addStatusMessage(errStr);
    }

    m_custom_session_code = code;

    if(code == 2001)
    {
        emit sigCreateSocket(m_customer_token);
    }
}


//会话创建完成数据传递  1000
void CustomerChatWidget::slotSessionStart(QByteArray data)
{

    m_custom_session_code = 1000;

    QJsonParseError json_error;
    QJsonDocument jsonDoc(QJsonDocument::fromJson(data.simplified(), &json_error));

    if(json_error.error != QJsonParseError::NoError)
    {
        qDebug() << "json error!";
    }

    QJsonObject rootObject = jsonDoc.object();

    QString sessionMsg = rootObject["message"].toString();

    QJsonObject jsonObject  = rootObject["assign_info"].toObject();

    QString agent_name = jsonObject["agent_name"].toString();
    QString iconpath = jsonObject["agent_avatar"].toString();
    m_session_id  = jsonObject["im_sub_session_id"].toDouble();

    QString downpath = downServiceFile(iconpath);

    m_pChatWidget->setServiceTitle(agent_name, downpath);
    m_pTitleWidget->setTitle(agent_name, downpath);

    //回话创建成功之后，发送消息传递custom_token
    emit sigCreateSocket(m_customer_token);

    //添加会话创建消息框显示
    m_pChatWidget->addStatusMessage(sessionMsg);
}

void CustomerChatWidget::postUnArrivedMessage()
{
    if(m_session_id == 0)
        return;

    if(m_pMessageUnArrived.count() > 0)
    {
        for(int i = 0; i < m_pMessageUnArrived.count(); i++)
        {
           Message message = m_pMessageUnArrived.at(i);

           if(! DNetState::getInstance()->getNetState())
           {
               QFont ft;
               ft.setPointSize(9);
               DFloatingMessage *msg = new DFloatingMessage();
               msg->setFont(ft);
               msg->setDuration(800);
               msg->setMessage( tr("network unavailable,message can not send Successful"));

               DMessageManager::instance()->sendMessage(m_pChatWidget, msg);

               return;
           }

           QJsonObject rootObj;

           rootObj.insert("customer_token", m_customer_token);
           rootObj.insert("im_sub_session_id",m_session_id);
           rootObj.insert("message_id",  message.message_id);
           rootObj.insert("type", message.type.toLower());

           QJsonObject dataobject;

           dataobject.insert("content", message.content);

           rootObj.insert("data", QJsonValue(dataobject));

           //发送文件消息
           emit signalMessage(QJsonDocument(rootObj).toJson().simplified());

        }

        m_pMessageUnArrived.clear();

    }
}


//用户消息回传
void CustomerChatWidget::slotRecvMessageRet(QByteArray data)
{

    QJsonParseError json_error;
    QJsonDocument jsonDoc(QJsonDocument::fromJson(data.simplified(), &json_error));

    if(json_error.error != QJsonParseError::NoError)
    {
        qDebug() << "json error!";
    }

    QJsonObject jsonObject = jsonDoc.object();

    QString msgid = jsonObject["message_id"].toString();

    m_pChatWidget->setMessageBack(msgid);
}


void CustomerChatWidget::onSocketCreated()
{
    postUnArrivedMessage();
}

//服务端消息回传回来时，出发添加服务消息槽函数
void CustomerChatWidget::slotRecvMessage(QByteArray msg)
{
    QJsonParseError json_error;
    QJsonDocument jsonDoc(QJsonDocument::fromJson(msg.simplified(), &json_error));

    if(json_error.error != QJsonParseError::NoError)
    {
        qDebug() << "json error!";
    }

    QJsonObject jsonObject = jsonDoc.object();

    QJsonArray array = jsonObject["messages"].toArray();

    double oldsessionid = m_session_id;
    double session_id = 0;
    for(int i = 0; i < array.size(); i++)
    {
        QJsonValue aryValue = array.at(i);
        if(aryValue.isObject())
        {
            QJsonObject obj = aryValue.toObject();
            QString type = obj["type"].toString();
            QString msgid = obj["message_id"].toString();
            QJsonObject msgdata = obj["data"].toObject();

            QString name = obj["agent_name"].toString();
            QString icon = obj["agent_avatar"].toString();

            session_id = obj["im_sub_session_id"].toDouble();

            Message message;

            message.name = name;

            QString downpath = downServiceFile(icon);

            message.icon = downpath;

            m_pChatWidget->setServiceTitle(message.name , downpath);
            m_pTitleWidget->setTitle(message.name , downpath);

            message.message_id = msgid;
            message.direct = "Receive";

            if(type == "image")  //发送路径
            {
                m_custom_session_code = 1000;
                QString content = msgdata["content"].toString();

                message.type = "Image";

                QString filepath = downServiceFile(content);
                message.content = filepath;
                m_pChatWidget->addMessage(message);
                m_session_id = session_id;
            }
            else if(type == "file") //发送路径
            {
                m_custom_session_code = 1000;

                message.type = "File";
                QString content = msgdata["content"].toString();
                QString filesize = msgdata["filesize"].toString();
                QString filename = msgdata["filename"].toString();

                QString filepath = downServiceFile(content);
                message.content = filepath;

                message.filename = filename;
                message.filesize = filesize;
                m_pChatWidget->addMessage(message);
                m_session_id = session_id;
            }
            else if(type == "message") //发送消息
            {
                m_custom_session_code = 1000;
                message.type = "Message";
                message.content = msgdata["content"].toString();
                m_pChatWidget->addMessage(message);
                m_session_id = session_id;
            }
            else if(type == "close")   //客服关闭会话
            {
                message.content = msgdata["content"].toString();

                m_session_id = 0;

                m_custom_session_code = 0;

                emit sigCloseWebSocket();

                //添加会话创建消息框显示
                m_pChatWidget->addStatusMessage(message.content);
            }
        }
    }

    //客户信息从客服排队中被接入处理．
    if((oldsessionid == 0) && (m_custom_session_code ==1000) && session_id != 0)
    {
        m_session_id = session_id;
        postUnArrivedMessage();
    }
}



