#ifndef CUSTOMERCHATWIDGET_H
#define CUSTOMERCHATWIDGET_H

#include <DWidget>

#include <DFrame>
#include <DScrollArea>
#include <DTextEdit>
#include <QList>
#include <DIconButton>
#include <DLabel>
#include <DCommandLinkButton>

DWIDGET_USE_NAMESPACE

class WebSocketClient;
class QVBoxLayout;
class QHBoxLayout;


typedef struct Message
{
    QString name;               //消息发送者名称
    QString icon;              //消息发送者头像
    QString datetime;          //消息时间
    QString type;              //消息类型 image,file,message
    QString direct;             //消息方向 Send Receive
    QString message_id;         //消息id  {fdsfasdf-fewfewfe-ewfwe-fewf-fewfwefwef}
    QString content;            //消息内容   消息内容，文件全路径
    QString filename;           //文件名称
    QString filesize;           //文件大小

}Message;

class TitleFrame : public DFrame
{
    Q_OBJECT
public:
    explicit TitleFrame(QWidget* parent = nullptr);
    virtual ~TitleFrame();

private:
    QString m_sCustomerName;
    QString m_sCustomerIcon;

    DLabel* m_picon;
    DLabel* m_plabel;

public:
    void setTitle(QString, QString);

};


class MessageFrame : public DFrame
{
    Q_OBJECT

public:

    explicit MessageFrame(Message msg, QWidget* parent = nullptr);

    virtual ~MessageFrame();

    void setMessageRecive();

    QString getMessageId();

    Message getMessage();

    DFrame* createFileFrame();

    DLabel* createImageLabel();

private:

    DMenu* m_pMenu;

    Message m_pCurMessage;

    QHBoxLayout* m_pMainLayout;

    QLabel* m_pMessageSuccessLabel;           //标志消息发送成功失败标志

    DLabel* m_pmsglabel;

    DFrame* m_pmsgFrame;

    bool m_bSendSuccess;

    DLabel* m_pTimeLabel;

    DLabel* m_pIconLabel;

    DLabel* m_pNameLabel;

    QTextEdit* m_pMessageEdit;

private:

    void initUI();

    void wrapData(const QString& text);

protected:

    bool eventFilter(QObject *o, QEvent *e);

private slots:

    void onSaveAsFile();

    void onOpenFile();

    void adjustContent();
};

class ChatFrame : public DScrollArea
{
    Q_OBJECT

public:
    explicit ChatFrame(QWidget* parent = nullptr);

    virtual ~ChatFrame();

    void setCustomTitle(QString name,QString icon);

    void setServiceTitle(QString name,QString icon);

    void setHistoryBtn(DCommandLinkButton*);

    void createMessageLog();  //创建日志文件

    //添加消息
    void addMessage(Message msg);

    //添加状态消息
    void addStatusMessage(QString);

    void addSessionStart(QString msg);

    void addSessionClose(QString msg);

    void insertHistroyMessages(QJsonArray messages);

    QJsonArray readMessages(QString file);

    //设置消息回传成功标志
    void setMessageBack(QString);

    void setCurEarlyDateTime(QString);

private:
    void saveMessage(Message msg);

    void writeLogFile(QByteArray data);

private:
    QVBoxLayout* m_pLayout;

    QScrollBar* m_pVerBar;

    DLabel *m_pHistoryLabel;

    QList<MessageFrame*> m_pMessages;

    QList<QString> m_pHistorys;

    int m_curHistoryIndex;

    QString m_pServerName;
    QString m_pServerIcon;

    QString m_pCustomIcon;
    QString m_pCustomName;

    QString m_CurEarlyDateTime;

    DCommandLinkButton  *m_pHistoryBtn;

public  slots:

    void onReadHistroy();
};


class CustomerChatWidget : public DFrame
{
    Q_OBJECT

public:
    explicit CustomerChatWidget(QWidget *parent = nullptr);
    ~CustomerChatWidget();

private:

   //界面UI类
   TitleFrame*  m_pTitleWidget;
   ChatFrame*   m_pChatWidget;
   QTextEdit*   m_pUserInputWidget;
   DLabel *     m_pEnterTipsLabel;
   DIconButton* m_pOpenFileButton;

   double       m_session_id;

   QString      m_customer_token;

   DCommandLinkButton  *m_pHistoryBtn;

   QString      m_icon;

   QString      m_name ;

   QString      m_user_uuid;

   int       m_custom_session_code;

   QList<Message> m_pMessageUnArrived;
private:
    //初始化界面UI
    void initUI();

    void getSystemIconName();  //获取系统图标和系统用户名

    void createSession();

    QString getCustomSesson();

    QString downServiceFile(QString urlpath);

    void sendMessage();

    void sendFile(QString);

    void closeSession();

    void postUnArrivedMessage();

signals:
    void signalUploadFile(QString, QString);

    //发送消息
    void signalMessage(QByteArray);

    //发送满意度
    void signalSurvey(QByteArray);

    //发送开始会话
    void sigStartSession(QByteArray);

    void sigCreateSocket(QString);

    void sigCloseWebSocket();
public slots:

    void onSocketCreated();
    //接受到客服消息
    void slotRecvMessage(QByteArray);

    //选择文件，发送文件
    void slotOpenFile();

    //会话失败
    void slotSessionFault(double code, QString errStr);

    //会话成功返回
    void slotSessionStart(QByteArray data);

    //用户消息回传
    void slotRecvMessageRet(QByteArray data);

    void slotUploadFileSuccess(QString weburl, QByteArray  data);

protected:
    bool eventFilter(QObject *o, QEvent *e);
};

#endif // CUSTOMERCHATWIDGET_H
