/*
 * Copyright (C) 2020 ~ 2020 Deepin Technology Co., Ltd.
 *
 * Author:     zhanglei <zhanglei_cm@deepin.com>
 *
 * Maintainer: zhanglei <zhanglei_cm@deepin.com>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "messageconsultationwidget.h"
#include <DTitlebar>
#include <QDBusInterface>
#include <QDebug>
#include <QMap>
#include <DFileDialog>
#include <QDialog>
#include <QCoreApplication>
#include <QMessageBox>
#include <QFileIconProvider>
#include <QTemporaryFile>
#include <QIcon>
#include <QJsonObject>
#include <QJsonDocument>
#include <QDBusReply>
#include <QDBusInterface>
#include <QStandardPaths>
#include <DMessageManager>
#include <QApplication>
#include <QFileInfo>
#include <QRegExp>
#include <cstdlib>
#include "dsysinfo.h"
#include "../../devicetree.h"
#include <stdio.h>
#include <string.h>
#include <systemd/sd-journal.h>
#include <QToolTip>
#include <DMessageManager>

DCORE_USE_NAMESPACE

#define FILE_SIZE (10*1024*1024)//单个附件的大小不超过10M
#define COUNT 5


EnclosureFrame::EnclosureFrame(QString filename, DFrame* parent ):DFrame(parent)
{
    QHBoxLayout* playot = new QHBoxLayout;
    QVBoxLayout* pvlayout = new QVBoxLayout;

    QFont ft;
    ft.setPointSize(9);

    m_sFileName =  filename;
    QFileInfo file_info(filename);
    QFileIconProvider icon_provider;
    QIcon icon = icon_provider.icon(file_info);

    pNameLabel = new DLabel;
    DLabel* psizelabel = new DLabel;

    pNameLabel->setFont(ft);
    psizelabel->setFont(ft);

    DLabel  *pIconLabel = new DLabel;
    pIconLabel->setFixedSize(26,26);

    pIconLabel->setPixmap(icon.pixmap(26,26));
    pIconLabel->setScaledContents(true);

    pNameLabel->setText(file_info.fileName());

    m_pCloseLabel = new DLabel(this);
    QIcon icon1(":/images/close.svg");

    m_pCloseLabel->setPixmap(icon1.pixmap(QSize(20,20)));

    qint64 filesize = file_info.size();

    QString sz;
    QString unit = "B";
    if(filesize > 1024) {
            filesize /= 1024;
            unit = "Kb";
            if(filesize > 1024) {
            filesize /= 1024;
            unit = "Mb";
            if(filesize > 1024) {
            filesize /= 1024;
            unit = "Gb";
            }
        }
    }

    sz.append(QString::number(filesize)).append(unit);

    psizelabel->setText(sz);

    pvlayout->addWidget(pNameLabel);
    pvlayout->setSpacing(2);
    pvlayout->addWidget(psizelabel);

    playot->addWidget(pIconLabel);

    playot->addLayout(pvlayout);

    playot->addStrut(5);

    playot->addWidget(m_pCloseLabel);

    playot->setStretchFactor(pIconLabel,2);
    playot->setStretchFactor(pvlayout,7);
    playot->setStretchFactor(m_pCloseLabel,1);

    playot->setSpacing(10);

    playot->setContentsMargins(11,5,11,5);

    this->setLayout(playot);

    this->setBackgroundRole(DPalette::ItemBackground);

    this->setFixedSize(230, 52);

    this->setContentsMargins(3,3,3,3);

     pNameLabel->installEventFilter(this);
     m_pCloseLabel->installEventFilter(this);
}

EnclosureFrame::~EnclosureFrame()
{

}

bool EnclosureFrame::eventFilter(QObject *o, QEvent *e)
{
    if(o == m_pCloseLabel)
    {
        if(e->type() == QEvent::MouseButtonRelease)
        {
            QMouseEvent* peve = dynamic_cast<QMouseEvent*>(e);
            if(peve != nullptr)
            {
              if(peve->button() == Qt::LeftButton)
              {
                  emit sigFrameInfo(this, m_sFileName);
              }
            }
        }
    }
    if(o == pNameLabel)
    {
        if (e->type() == QEvent::Enter)
        {
            this->setToolTip(pNameLabel->text());
            //qDebug()<<"Enter";
        }
    }
    return DFrame::eventFilter(o,e);
}

MessageConsultation::MessageConsultation(QWidget *parent)
    : DWidget(parent)
{
    initUI();

    this->setAcceptDrops(true);
    QFont ft;
    ft.setPointSize(9);

    popDlg = new DDialog(this);
    popDlg->setFont(ft);
    popDlg->addButton(tr("sure"));
    popDlg->setIcon(QIcon(":/images/deepin-cloudprint-config-helper.svg"));

    connect(m_addEnclosure_cmdlnkbtn, &DCommandLinkButton::clicked, this, &MessageConsultation::addEnclosureResp);

    //open log
    connect(m_log_switchBtn,  &DSwitchButton::clicked, this,
            [=](bool bFlag)
    {
        if(bFlag)
        {
            m_upload_label->setText(tr("Uploading"));
            m_upload_label->setFont(ft);
            if(m_log_spinner->isHidden())
            {
                m_log_spinner->show();
                m_log_spinner->start();
            }

            if(readLog() == 0)
            {
                m_log_spinner->hide();
                m_consultationImage_label->show();
                m_upload_label->setText(tr("Logs:"));
                m_log_hBoxLayout->addWidget(m_consultationImage_label);
            };

            QString savename = "user_report.log";

            QString savepath = QStandardPaths::writableLocation(QStandardPaths::AppLocalDataLocation) + "/user_collect_log/";

            QString filepath = savepath + savename;

            m_filePathLists.push_back(filepath);

        }
        else
        {
            m_upload_label->setText("");
            m_log_spinner->hide();
            m_consultationImage_label->hide();


            QString savename = "user_report.log";

            QString savepath = QStandardPaths::writableLocation(QStandardPaths::AppLocalDataLocation) + "/user_collect_log/";

            QString filepath = savepath + savename;

            m_filePathLists.removeOne(filepath);

        }
    });

    connect(m_commit_SuggestBtn,  &DSuggestButton::clicked, this,
            [=]()
    {
        QString strConsultationType_1 = m_consultationType_1->currentText();
        QString strConsultationType_2 = m_consultationType_2->currentText();
        QString strConsultationType_3 = m_consultationType_3->currentText();
        QString strTitle = m_title_lineEdit->text();
        QString strContent = m_content_textEdit->toPlainText();

        QString strEMail = m_mail_lineEdit->text();

        if(strTitle.isEmpty() && strContent.isEmpty() && strEMail.isEmpty())
        {
            popDlg->setMessage(tr("The title, content and email are required, please fill in and submit it again"));
            popDlg->exec();
            return;
        }
        if(strTitle.isEmpty() && strContent.isEmpty() && (strEMail.isEmpty() == false))
        {
            popDlg->setMessage(tr("The title and content are required, please fill in and submit it again"));
            popDlg->exec();
            return;
        }
        if((strContent.isEmpty()) && strEMail.isEmpty() && (strTitle.isEmpty() == false))
        {
            popDlg->setMessage(tr("The content and email are required, please fill in and submit it again"));
            popDlg->exec();
            return;
        }
        if((strTitle.isEmpty()) && strEMail.isEmpty() && (strContent.isEmpty() == false))
        {
            popDlg->setMessage(tr("The title and email are required, please fill in and submit it again"));
            popDlg->exec();
            return;
        }
        if((strTitle.isEmpty()) &&(strContent.isEmpty() == false) && (strEMail.isEmpty() == false))
        {
            popDlg->setMessage(tr("Title are required, please fill in and submit again"));
            popDlg->exec();
            return;
        }
        if((strContent.isEmpty()) &&(strTitle.isEmpty() == false) && (strEMail.isEmpty() == false))
        {
            popDlg->setMessage(tr("Content are required, please fill in and submit again"));
            popDlg->exec();
            return;
        }
        if((strEMail.isEmpty()) &&(strTitle.isEmpty() == false) && (strContent.isEmpty() == false))
        {
            popDlg->setMessage(tr("Email are required, please fill in and submit again"));
            popDlg->exec();
            return;
        }

        QRegExp pattern("[0-9a-zA-Z]+[@]+[0-9a-zA-Z]+[\.]+[0-9a-zA-Z]+$");

        if (!pattern.exactMatch( m_mail_lineEdit->text()))
        {
            m_mail_lineEdit->showAlertMessage(tr("The email address is incorrect"),m_commit_SuggestBtn,3000);
            return;
        }

        // 构建 Json 对象,内容为创建工单的基础信息
        QJsonObject tickerObj;
        tickerObj.insert("subject", strTitle);
        tickerObj.insert("content", strContent);
        tickerObj.insert("creator_email", strEMail);

        QJsonObject categoryObj;
        categoryObj.insert("system", strConsultationType_1 );
        categoryObj.insert("block", strConsultationType_2 );
        categoryObj.insert("item", strConsultationType_3);

        //如果没有网，直接跳转页面
        if(!DNetState::getInstance()->getNetState())
        {
            DDialog *popDlg = new DDialog(this);;
            popDlg->setIcon(QIcon(":/images/deepin-cloudprint-config-helper.svg"));
            popDlg->setMessage(tr("Network exception, please check the network before submitting"));
            popDlg->addButton(tr("sure"));
            popDlg->setFont(ft);
            popDlg->exec();
            return;
        }

        //发送数据
         emit signalTicket(getUserName(), QJsonDocument(tickerObj).toJson().simplified(),
                           QJsonDocument(categoryObj).toJson().simplified(), m_filePathLists);
    });

    connect(m_reset_pushBtn,  &DPushButton::clicked, this,
            [=]()
    {
        clearSendContent();
    });

    //初始化system combobox
    m_consultationType_1->addItems(devicetree::getInstance()->getSystemNames());
    //设置system combox 默认值为第一项，联动出发信号改变
    m_consultationType_1->setCurrentIndex(0);


}

void MessageConsultation::slotPostFaild()
{
    QFont ft;
    ft.setPointSize(9);
    DDialog *popDlg = new DDialog(this);
    popDlg->setIcon(QIcon(":/images/deepin-cloudprint-config-helper.svg"));
    popDlg->setMessage(tr("The server connection is abnormal. Please try again later"));
    popDlg->addButton(tr("sure"));
    popDlg->setFont(ft);
    popDlg->exec();
    return;
}

void MessageConsultation::reGeometryLayout()
{
    QList<QLayoutItem* > pitems;

    for(int i = 0; i < m_enclosure_gridLayout->count();i++)
    {
        QLayoutItem* item  = m_enclosure_gridLayout->itemAt(i);
        pitems.push_back(item);
    }

    for(int i = 0; i < m_enclosure_gridLayout->count();i++)
    {
        QLayoutItem* item  = m_enclosure_gridLayout->itemAt(i);
        m_enclosure_gridLayout->removeItem(item);
    }
    //grid layout 重新佈局
    for(int i = 0; i < pitems.size();i++)
    {
        int row = (i) / 2.0;

        int column = i % 2;

        //setParent为NULL，防止删除之后界面不消失
        if(pitems.at(i)->widget())
        {
            //布局中添加控件
            m_enclosure_gridLayout->addWidget(pitems.at(i)->widget(), row, column);
        }
    }
}

void MessageConsultation::deleteFrame(EnclosureFrame* frame, QString text)
{
    if(!m_uploadexp_label->text().isEmpty())
    {
        m_uploadexp_label->clear();

        m_enclosure_gridLayout->removeWidget(m_uploadexp_label);
    }

    if(frame != nullptr)
    {
        m_enclosure_gridLayout->removeWidget(frame);
        m_filePathLists.removeOne(text);
        frame->deleteLater();
    }
   // reGeometryLayout();
}

void MessageConsultation::slotUpdateProgress(QString str)
{
    //qDebug()<<"progress..."<<str;
    if((str == "1") || (str == "nan"))
    {
        m_commit_SuggestBtn->setEnabled(true);
        m_reset_pushBtn->setEnabled(true);
    }
    else {
        m_commit_SuggestBtn->setEnabled(false);
        m_reset_pushBtn->setEnabled(false);
    }


}

void MessageConsultation::slotTicketComplete(qint64 n_tickId)
{
    QFont ft;
    ft.setPointSize(9);
    if(n_tickId !=0)
    {
        clearSendContent();
    }
    else {
        DDialog *popDlg = new DDialog;
        popDlg->setIcon(QIcon(":/images/deepin-cloudprint-config-helper.svg"));
        popDlg->setMessage(tr("The server connection is abnormal. Please try again later"));
        popDlg->addButton(tr("sure"));
        popDlg->setFont(ft);
        popDlg->exec();
    }

}

void MessageConsultation::clearSendContent()
{
    m_title_lineEdit->clear();
    m_content_textEdit->clear();
    m_mail_lineEdit->clear();
    m_uploadexp_label->clear();
    m_log_switchBtn->setChecked(false);
    m_upload_label->clear();
    m_consultationImage_label->hide();
    slotUpdateComboBox();

    m_filePathLists.clear();//清除附件列表
    //清空附件附件布局中的所有元素
    QLayoutItem *child;

    while ((child = m_enclosure_gridLayout->takeAt(0)) != 0)
    {
        //setParent为NULL，防止删除之后界面不消失
        if(child->widget())
        {
            child->widget()->setParent(nullptr);
        }
        delete child;
    }
}

MessageConsultation::~MessageConsultation()
{
}

int MessageConsultation::readLog()
{

    QString oldpath = QDir::currentPath();

    QString savename = "user_report.log";

    QString savepath = QStandardPaths::writableLocation(QStandardPaths::AppLocalDataLocation) + "/user_collect_log/";

    QDir logDir;

    if(!logDir.exists(savepath))
    {
        logDir.mkpath(savepath);
    }

    //将程序的执行路径设置到filePath下
    logDir.setCurrent(savepath);

    QFile writefile(savename);

    if(!writefile.open(QIODevice::WriteOnly | QIODevice::Truncate))
    {
        qDebug()<<tr("open file fail");
    }

    int r;
    sd_journal *j;
    r = sd_journal_open(&j, SD_JOURNAL_CURRENT_USER | SD_JOURNAL_SYSTEM);
    if (r < 0) {
        fprintf(stderr, "Failed to open journal: %s\n", strerror(-r));
        logDir.setCurrent(oldpath);
        return 1;
    }

    uint64_t from = QDateTime::currentDateTime().addDays(-2).toMSecsSinceEpoch();

    uint64_t to = QDateTime::currentMSecsSinceEpoch();

    r = sd_journal_get_cutoff_realtime_usec(j, &from, &to);
    SD_JOURNAL_FOREACH(j) {
        const char *d;
        size_t l;

        r = sd_journal_get_data(j, "MESSAGE", (const void **)&d, &l);

        if (r < 0) {
           fprintf(stderr, "Failed to read message field: %s\n", strerror(-r));
           logDir.setCurrent(oldpath);
           continue;
        }

        printf("%s\n", d);
        if(m_log_switchBtn->isChecked())
        {

            writefile.write(d);
        }
        else {
            sd_journal_close(j);

            writefile.close();

            logDir.setCurrent(oldpath);
            return -1;
        }

        QApplication::processEvents();
    }
    sd_journal_close(j);

    writefile.close();

    logDir.setCurrent(oldpath);

    return 0;

}

void MessageConsultation::initUI()
{
    QFont ft2;
    ft2.setPointSize(9);
    QScrollArea* scrollArea = new QScrollArea();
    scrollArea->setHorizontalScrollBarPolicy(Qt::ScrollBarAlwaysOff);
    QWidget* scrollAreaWidgetContents = new QWidget;
    scrollArea->setWidget(scrollAreaWidgetContents);

    QVBoxLayout* mainlayout = new QVBoxLayout;
    QVBoxLayout* scroll_vBoxLayout = new QVBoxLayout;

    DFrame* consultationType_frame = new DFrame;
    consultationType_frame->setBackgroundRole(DPalette::ItemBackground);
    QHBoxLayout* consultationType_hBoxLayout = new QHBoxLayout;
    DLabel* consultationType_label = new DLabel;
    m_consultationType_1 = new DComboBox;
    m_consultationType_1->setFont(ft2);
    m_consultationType_2 = new DComboBox;
    m_consultationType_2->setFont(ft2);
    m_consultationType_3 = new DComboBox;
    m_consultationType_3->setFont(ft2);
    consultationType_label->setFixedSize(60,29);

    m_consultationType_1->setFixedSize(164, 36);
    m_consultationType_2->setFixedSize(164, 36);
    m_consultationType_3->setFixedSize(164, 36);
    consultationType_hBoxLayout->addWidget(consultationType_label);
    consultationType_hBoxLayout->addWidget(m_consultationType_1);
    consultationType_hBoxLayout->addWidget(m_consultationType_2);
    consultationType_hBoxLayout->addWidget(m_consultationType_3);
    consultationType_frame->setLayout(consultationType_hBoxLayout);


    consultationType_label->setText(tr("Category"));
    consultationType_label->setFont(ft2);

    DFrame* title_frame = new DFrame;
    title_frame->setBackgroundRole(DPalette::ItemBackground);
    QHBoxLayout* title_hBoxLayout = new QHBoxLayout;
    DLabel* title_label = new DLabel;

    DLabel *Must_title = new DLabel("*");
    QPalette Title_pe;
    Must_title->setFont(ft2);
    Title_pe.setColor(QPalette::Text,Qt::red);
    Must_title->setPalette(Title_pe);
    m_title_lineEdit = new QLineEdit;

    //信号槽链接
    QObject::connect(m_consultationType_1, &QComboBox::currentTextChanged, \
                     this, &MessageConsultation::onSystemComboBoxIndexChanged);
    QObject::connect(m_consultationType_2,&QComboBox::currentTextChanged, \
                     this, &MessageConsultation::onBlockComboBoxIndexChanged);
    QObject::connect(m_consultationType_3, &QComboBox::currentTextChanged, \
                     this, &MessageConsultation::onItemComboBoxIndexChanged);

    title_hBoxLayout->addWidget(title_label);
    title_hBoxLayout->addWidget(Must_title);
    title_hBoxLayout->addWidget(m_title_lineEdit);
    m_title_lineEdit->setFixedSize(485, 36);
    title_frame->setLayout(title_hBoxLayout);

    title_label->setText(tr("Title"));
    title_label->setFont(ft2);
    m_title_lineEdit->setPlaceholderText(tr("Input the title you want to consult or suggest"));
    m_title_lineEdit->setFont(ft2);
    m_title_lineEdit->setClearButtonEnabled(true);
    m_title_lineEdit->setMaxLength(31);
    m_title_lineEdit->setAcceptDrops(false);

    connect(m_title_lineEdit, &QLineEdit::textChanged, this,
            [ = ](const QString & keyText) {
            if (keyText.size() == 31) {
                QString text = keyText.left(keyText.length() - 1);
                m_title_lineEdit->setText(text);
            
            DDialog *popDlg = new DDialog(this) ;
                popDlg->setIcon(QIcon(":/images/deepin-cloudprint-config-helper.svg"));
                popDlg->setMessage(tr("The title should be no more than 30 characters"));
                popDlg->addButton(tr("sure"));
                popDlg->setFont(ft2);
                popDlg->exec();
                return;
            }
    });
    DFrame* content_frame = new DFrame;
    content_frame->setAcceptDrops(true);
    content_frame->setBackgroundRole(DPalette::ItemBackground);
    QVBoxLayout* content_vLayout = new QVBoxLayout;
    QHBoxLayout* content1_hLayout = new QHBoxLayout;

    DLabel *Must_content = new DLabel("*");
    QPalette content_pe;
    Must_content->setFont(ft2);
    content_pe.setColor(QPalette::Text,Qt::red);
    Must_content->setPalette(content_pe);

    DLabel* content_label = new DLabel(tr("Content"));
    content_label->setFont(ft2);
    m_content_textEdit = new DTextEdit;

    m_content_textEdit->setPlaceholderText(tr("* Please write the detailed description\n"
                                              "\n"
                                              "* You can attach screenshots, videos, and GIFs (using Screen Capture)"));
    m_content_textEdit->setFont(ft2);
    m_content_textEdit->setAcceptDrops(false);
    m_content_textEdit->setFixedSize(485, 119);

    content1_hLayout->addWidget(content_label, 0, Qt::AlignTop);
    content1_hLayout->addWidget(Must_content, 0, Qt::AlignTop);
    content1_hLayout->addWidget(m_content_textEdit, 0, Qt::AlignTop);

    QHBoxLayout* content2_hLayout = new QHBoxLayout;
    m_contentCount_label = new DLabel("0/200");
    m_contentCount_label->setFont(ft2);
    m_addEnclosure_cmdlnkbtn = new DCommandLinkButton(tr("text"));
    DLabel* enclosureSize_label = new DLabel(tr("(Each one ≤ 10 MB)"));
    QPalette color_enclosure;
    color_enclosure.setColor(QPalette::Text,QColor(82,106,127,255));
    enclosureSize_label->setFont(ft2);
    enclosureSize_label->setPalette(color_enclosure);
    m_addEnclosure_cmdlnkbtn->setText(tr("Add attachments "));
    m_addEnclosure_cmdlnkbtn->setFont(ft2);

    content2_hLayout->addSpacing(90);
    content2_hLayout->addWidget(m_contentCount_label);
    content2_hLayout->addSpacing(280);
    content2_hLayout->addWidget(m_addEnclosure_cmdlnkbtn);
    content2_hLayout->addWidget(enclosureSize_label);
    content2_hLayout->addSpacing(30);

    connect(m_content_textEdit, &QTextEdit::textChanged, this,
            [=]()
    {
        QString textContent = m_content_textEdit->toPlainText();
           int length = textContent.count();
           int maxLength = 200; // 内容区最大字符数
           QString num =  QString::number(length);
           m_contentCount_label->setText(num+"/200");
           m_contentCount_label->setFont(ft2);
           if(length > maxLength)
           {
               int position = m_content_textEdit->textCursor().position();
               QTextCursor textCursor = m_content_textEdit->textCursor();
               textContent.remove(position - (length - maxLength), length - maxLength);
               m_content_textEdit->setText(textContent);
               textCursor.setPosition(position - (length - maxLength));
               m_content_textEdit->setTextCursor(textCursor);
           }
    });

    content_vLayout->addLayout(content1_hLayout);
    content_vLayout->addLayout(content2_hLayout);
    content_frame->setLayout(content_vLayout);

    m_uploadexp_label = new DLabel;
    m_uploadexp_label->setStyleSheet("color:#FF5736;");
    m_uploadexp_label->setText(tr(""));

    m_enclosure_gridLayout = new QGridLayout;

    m_enclosure_gridLayout->setHorizontalSpacing(10);
    m_enclosure_gridLayout->setVerticalSpacing(10);

    content_vLayout->addLayout(m_enclosure_gridLayout);

    m_enclosure_gridLayout->setContentsMargins(66,0,0,0);

    m_enclosure_gridLayout->setAlignment(Qt::AlignRight | Qt::AlignTop);

    DFrame* log_frame = new DFrame;
    log_frame->setBackgroundRole(DPalette::ItemBackground);
    log_frame->setFixedSize(570, 48);
    m_log_hBoxLayout = new QHBoxLayout;

    DLabel* log_label = new DLabel(tr("Upload log"));
    log_label->setFont(ft2);

    m_log_switchBtn = new DSwitchButton;
    m_upload_label = new DLabel(tr(""));
    m_consultationImage_label = new DLabel;
    QImage image;
    image.load(":/images/compress.svg");
    m_consultationImage_label->setPixmap(QPixmap::fromImage(image));
    m_log_spinner = new DSpinner;
    m_log_spinner->hide();

    m_log_spinner->setFixedSize(16, 16);

    m_log_hBoxLayout->addWidget(log_label);
    m_log_hBoxLayout->addSpacing(150);
    m_log_hBoxLayout->addWidget(m_log_switchBtn);
    m_log_hBoxLayout->addSpacing(156);
    m_log_hBoxLayout->addWidget(m_upload_label);
    m_log_hBoxLayout->addWidget(m_log_spinner);
    m_log_hBoxLayout->addSpacing(10);

    log_frame->setLayout(m_log_hBoxLayout);

    QLabel* logExplain_label = new QLabel;
    QFont ft1;
    ft1.setPointSize(8);
    QPalette color_lable;
    color_lable.setColor(QPalette::Text,QColor(82,106,127,255));
    QString str = tr("Your personal information will not be collected when uploading logs");
    logExplain_label->setFont(ft1);
    logExplain_label->setText(str);
    logExplain_label->setFixedHeight(18);
    logExplain_label->setPalette(color_lable);
    DLabel *Must_mail = new DLabel("*");
    QPalette Mail_pe;
    Must_mail->setFont(ft1);
    Mail_pe.setColor(QPalette::Text,Qt::red);
    Must_mail->setPalette(Mail_pe);

    DFrame* mail_frame = new DFrame;

    QHBoxLayout* mail_hBoxLayout = new QHBoxLayout;
    DLabel* mail_label = new DLabel(tr("Email"));
    mail_label->setFont(ft2);

    m_mail_lineEdit = new DLineEdit;
    m_mail_lineEdit->setFixedSize(485, 36);
    m_mail_lineEdit->setAcceptDrops(false);

    mail_hBoxLayout->addWidget(mail_label);
    mail_hBoxLayout->addWidget(Must_mail);
    mail_hBoxLayout->addWidget(m_mail_lineEdit);
    mail_frame->setLayout(mail_hBoxLayout);

    m_mail_lineEdit->lineEdit()->setPlaceholderText(tr("Write down your email to get the progress timely"));
    m_mail_lineEdit->setClearButtonEnabled(true);
    m_mail_lineEdit->setFont(ft2);

//    QRegExp regxName("^[A-Za-z0-9\u4e00-\u9fa5]+@[a-zA-Z0-9_-]+(\.[a-zA-Z0-9_-]+)+$");//邮箱的正则表达式
//    QRegExp regxName("^[a-zA-Z0-9_-]+@[a-zA-Z0-9_-]+(\.[a-zA-Z0-9_-]+)+$");
//    QValidator *validatorName = new QRegExpValidator(regxName,this);
//    m_mail_lineEdit->setValidator(validatorName);

    QHBoxLayout* useHelp_hLayout = new QHBoxLayout;
    DLabel* useHelp_label = new DLabel(tr("Instructions:"));
    useHelp_label->setFont(ft2);

    DLabel* useHelpContent_label = new DLabel;
    useHelpContent_label->setText(tr("1. Please ask for services and technical support by classifying your issue; \n"
                               "2. You also can upload the attachments by dragging and dropping; \n"
                               "3. Attachments ≤ 4 pcs, Each one ≤ 10 MB;\n"
                               "4. You will get the progress and results from UOS support team in your mailbox. ")
                            );
    useHelpContent_label->setFont(ft2);
    useHelpContent_label->setPalette(color_lable);
    useHelpContent_label->setFixedWidth(490);
    useHelp_hLayout->addWidget(useHelp_label, 0, Qt::AlignTop);
    useHelp_hLayout->addWidget(useHelpContent_label,  0, Qt::AlignTop);
    useHelp_hLayout->addSpacing(40);

    QHBoxLayout* btn_hLayout = new QHBoxLayout;
    m_commit_SuggestBtn = new DSuggestButton(tr("Submit"));
    m_commit_SuggestBtn->setFont(ft2);
    m_reset_pushBtn = new DPushButton(tr("Reset"));
    m_reset_pushBtn->setFont(ft2);
    m_commit_SuggestBtn->setFixedSize(170, 36);
    m_reset_pushBtn->setFixedSize(170, 36);
    btn_hLayout->addStretch(0);
    btn_hLayout->addWidget(m_reset_pushBtn);
    btn_hLayout->addWidget(m_commit_SuggestBtn);
    btn_hLayout->addStretch(0);


    consultationType_frame->setBackgroundRole(DPalette::ItemBackground);
    log_frame->setBackgroundRole(DPalette::ItemBackground);
    mail_frame->setBackgroundRole(DPalette::ItemBackground);

    consultationType_frame->setFixedSize(570, 48);
    content_frame->setMaximumSize(570, 305);
    title_frame->setFixedSize(570, 48);
    title_frame->setMaximumSize(570, 305);
    log_frame->setFixedSize(570,48);
    mail_frame->setFixedSize(570, 48);
    scroll_vBoxLayout->addWidget(consultationType_frame);
    scroll_vBoxLayout->addSpacing(6);
    scroll_vBoxLayout->addWidget(title_frame);
    scroll_vBoxLayout->addSpacing(6);
    scroll_vBoxLayout->addWidget(content_frame);
    scroll_vBoxLayout->addSpacing(6);
    scroll_vBoxLayout->addWidget(log_frame);
    scroll_vBoxLayout->addSpacing(2);
    scroll_vBoxLayout->addWidget(logExplain_label);
    scroll_vBoxLayout->addSpacing(2);
    scroll_vBoxLayout->addWidget(mail_frame);
    scroll_vBoxLayout->addSpacing(4);
    scroll_vBoxLayout->addLayout(useHelp_hLayout);

    scrollArea->setFrameShape(QFrame::NoFrame);
    scrollArea->widget()->setLayout(scroll_vBoxLayout);
    mainlayout->setMargin(0);
    mainlayout->addWidget(scrollArea);
    mainlayout->addLayout(btn_hLayout);

    scrollArea->setWidgetResizable(true);
    setLayout(mainlayout);
}


void MessageConsultation::addEnclosureResp()
{

    QFont ft;
    ft.setPointSize(9);
    DFileDialog fileDlg(this);//创建对话框
    //设置默认文件路径
    QString strDir = QDir::homePath();

    fileDlg.setDirectory(strDir);
    //设置可以选择多个文件,默认为只能选择一个文件QFileDialog::ExistingFilesCancel
    fileDlg.setFileMode(DFileDialog::ExistingFiles);

    //设置视图模式
    fileDlg.setViewMode(DFileDialog::Detail);

    QStringList fileNames;
    if (fileDlg.exec())
       fileNames = fileDlg.selectedFiles();

    int count = fileNames.size() + m_filePathLists.count();

    if (m_log_switchBtn->isChecked())
    {
         count--;
    }

    if (count > 4)
    {
       QFont ft2;
       ft2.setPointSize(9);
       DDialog *popDlg = new DDialog(this);
       popDlg->setIcon(QIcon(":/images/deepin-cloudprint-config-helper.svg"));
       popDlg->setMessage(tr("Attachments ≤ 4 pcs"));
       popDlg->addButton(tr("sure"));
       popDlg->setFont(ft2);
       popDlg->exec();
      // m_uploadexp_label->setText(tr("Attachments ≤ 4 pcs"));
       return;
    }
    handleFile(fileNames);
}


void MessageConsultation::handleFile(QStringList files)
{
    QFont ft;
    ft.setPointSize(9);
    //再次添加清除label內容和销毁该布局
    if(!m_uploadexp_label->text().isEmpty())
    {
        m_uploadexp_label->clear();

        m_enclosure_gridLayout->removeWidget(m_uploadexp_label);
    }

    int startcount = m_filePathLists.count() ;

    int j = 0;
    for(int i = 0; i < files.count(); i++)
    {
        QString file = files.at(i);

        QFileInfo info(file);

        if( info.size() > FILE_SIZE)
        {
            j++;
            continue;
        }

        if( info.size() == 0)
        {
            j++;
            continue;
        }

        EnclosureFrame* pframe = new EnclosureFrame(file);

        connect(pframe, &EnclosureFrame::sigFrameInfo, this, &MessageConsultation::deleteFrame);

        int count =  startcount +  i + 1 - j;

        if (m_log_switchBtn->isChecked())
        {
             count--;
        }

        int row = (count-1) / 2.0;

        int column = count % 2;

        //布局中添加控件
        m_enclosure_gridLayout->addWidget(pframe, row, column);

        reGeometryLayout();

        m_filePathLists.push_back(file);
    }

    for(int i = 0; i < files.count(); i++)
    {
        QString file = files.at(i);

        QFileInfo info(file);

        if( info.size() > FILE_SIZE)
        {
            m_uploadexp_label->setText(tr("Each one ≤ 10 MB"));
            m_enclosure_gridLayout->addWidget(m_uploadexp_label, 2, 1, Qt::AlignRight | Qt::AlignTop);
        }

        if( info.size() == 0)
        {
            m_uploadexp_label->setText(tr("Each one size cannot be zero"));
            m_enclosure_gridLayout->addWidget(m_uploadexp_label, 2, 1, Qt::AlignRight | Qt::AlignTop);//不能拖拽大小为0的附件
        }
    }

}

void MessageConsultation::dragEnterEvent(QDragEnterEvent *event)
{
    if(event->mimeData()->hasFormat("text/uri-list"))
    {
      QList<QUrl> urls =  event->mimeData()->urls();

      foreach(QUrl url, urls)
      {
          QFileInfo fi(url.toLocalFile());
          if(fi.isDir())//屏蔽文件夹，不支持拖拽文件夹
          {
              event->ignore();
              return;
          }
      }
      event->acceptProposedAction();
    }
    else
    {
       event->ignore();
    }
}

void MessageConsultation::dropEvent(QDropEvent *event)
{
    QList<QUrl> urls = event->mimeData()->urls();
    if(urls.isEmpty())
    {
         return;
    }

    int count = urls.size() + m_filePathLists.count();

    if (m_log_switchBtn->isChecked())
    {
         count--;
    }

    if(count > 4)
    {
        DDialog *popDlg = new DDialog(this);
        popDlg->setIcon(QIcon(":/images/deepin-cloudprint-config-helper.svg"));
        popDlg->setMessage(tr("Attachments ≤ 4 pcs"));
        popDlg->addButton(tr("sure"));

        popDlg->exec();
        return;
    }

    QStringList files;

    foreach(QUrl url, urls)
    {
        files.push_back(url.toLocalFile());
    }

    handleFile(files);
}

QStringList MessageConsultation::getFilePaths()
{
    return m_filePathLists;
}

//获得系统版本
QString MessageConsultation::getSysVer()
{
    QString version;

    if (DSysInfo::isDeepin()) {
        version = QString("%1 %2").arg(DSysInfo::deepinVersion())
                                  .arg(DSysInfo::deepinTypeDisplayName());
    } else {
        version = QString("%1 %2").arg(DSysInfo::productTypeString())
                                  .arg(DSysInfo::productVersion());
    }
    return version;
}
// 获得系统架构
QString MessageConsultation::getSysArch()
{
    QString strSysArch = "x64";

    return strSysArch;
}
//获得激活状态
quint32 MessageConsultation::getActiveState()
{
    QDBusInterface licenseInfo("com.deepin.license.activator",
                               "/com/deepin/license/activator",
                               "com.deepin.license.activator",
                               QDBusConnection::sessionBus());
    if (!licenseInfo.isValid()) {
        qWarning()<< "com.deepin.license error ,"<< licenseInfo.lastError().name();
    }

    QDBusReply<quint32> reply = licenseInfo.call(QDBus::AutoDetect,
                                   "GetIndicatorData");
    return reply;
}

//获得激活码
QString MessageConsultation::getActiveCode()
{
    QString activeCode= "1111-1111-1111-1111";
    return activeCode;
}

QString MessageConsultation::getUserName()
{
    QDBusInterface manual("com.deepin.daemon.Accounts",
                             "/com/deepin/daemon/Accounts/User1000",
                             "com.deepin.daemon.Accounts.User",
                             QDBusConnection::systemBus());

    QString name = manual.property("UserName").toString();

    return name;
}

void MessageConsultation::slotUpdateComboBox()
{
    m_consultationType_1->clear();
    m_consultationType_1->addItems(devicetree::getInstance()->getSystemNames());
    m_consultationType_1->setCurrentIndex(0);
}


void MessageConsultation::onSystemComboBoxIndexChanged(QString value)
{
    QStringList list  = devicetree::getInstance()->getBlockNames(value);
    m_consultationType_2->clear();
    m_consultationType_2->addItems(list);
    m_consultationType_2->setCurrentIndex(0);

}

void MessageConsultation::onBlockComboBoxIndexChanged(QString value)
{
    QStringList list  = devicetree::getInstance()->getItemNames(value);
    m_consultationType_3->clear();
    m_consultationType_3->addItems(list);
    m_consultationType_3->setCurrentIndex(0);
}

void MessageConsultation::onItemComboBoxIndexChanged(QString value)
{
    //修改完combobox，发送消息到服务器读取企业客服信息
}

