/*
 * Copyright (C) 2020 ~ 2020 Deepin Technology Co., Ltd.
 *
 * Author:     zhanglei <zhanglei_cm@deepin.com>
 *
 * Maintainer: zhanglei <zhanglei_cm@deepin.com>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef MESSAGECONSULTATION_H
#define MESSAGECONSULTATION_H

#include <DWidget>
#include <DScrollArea>
#include <QHBoxLayout>
#include <QVBoxLayout>
#include <QGridLayout>
#include <DFrame>
#include <DLabel>
#include <DComboBox>
#include <QLineEdit>
#include <DLineEdit>
#include <DTextEdit>
#include <DSwitchButton>
#include <DSpinner>
#include <DSuggestButton>
#include <DPushButton>
#include <DScrollArea>
#include <DListView>
#include <QStandardItemModel>
#include <DCommandLinkButton>
#include <QIcon>
#include <DDialog>
#include <QFont>
#include <QDragEnterEvent>
#include <QDropEvent>
#include <QMimeData>
#include <QUrl>
#include <DProgressBar>

#include "../../clientsocket.h"
#include "unit.h"

DWIDGET_USE_NAMESPACE

class EnclosureFrame : public DFrame
{
    Q_OBJECT
public:
    EnclosureFrame(QString , DFrame* parent = nullptr);
    ~EnclosureFrame();
protected:
    bool eventFilter(QObject *o, QEvent *e);
signals:
    void sigFrameInfo(EnclosureFrame *, QString);
private:
     DLabel* pNameLabel;
     DLabel* m_pCloseLabel;
     QString m_sFileName;
};

class MessageConsultation : public DWidget
{
    Q_OBJECT
public:
    explicit MessageConsultation(QWidget *parent = nullptr);
    ~MessageConsultation();

private:

    DDialog *popDlg;

    //combox 设置
    DComboBox *m_consultationType_1;
    DComboBox *m_consultationType_2;
    DComboBox *m_consultationType_3;

    QLineEdit *m_title_lineEdit;
    DTextEdit *m_content_textEdit;

    DLabel *m_contentCount_label;
    DCommandLinkButton *m_addEnclosure_cmdlnkbtn;

    QGridLayout *m_enclosure_gridLayout;

    DLabel *m_uploadexp_label;

    QHBoxLayout *m_log_hBoxLayout;

    DSwitchButton *m_log_switchBtn;

    QLabel *m_upload_label;
    DSpinner *m_log_spinner;
    DLabel *m_consultationImage_label;

    DLineEdit *m_mail_lineEdit;

    DSuggestButton *m_commit_SuggestBtn;
    DPushButton *m_reset_pushBtn;

    int length;

    //服务器连接状态
    bool m_bConnected;

    QStringList m_filePathLists;

signals:
    void signalTicket(const QString , const QByteArray , const QByteArray , QStringList );
    void ticketCreatSuccess();

private slots:
    // 服务器信息返回处理

    void addEnclosureResp();

    void onSystemComboBoxIndexChanged(QString);
    void onBlockComboBoxIndexChanged(QString);
    void onItemComboBoxIndexChanged(QString);

public slots:
   //设置故障树列表
    void slotUpdateComboBox();
    //工单创建成功
    void slotTicketComplete(qint64 n_tickId);
    //更新上传进度
    void slotUpdateProgress(QString);
    //post失败
    void slotPostFaild();

    void deleteFrame(EnclosureFrame*, QString);
protected:

    void dragEnterEvent(QDragEnterEvent *event);//拖动进入事件
    void dropEvent(QDropEvent *event);

private:
    void handleFile(QStringList);

    void reGeometryLayout();

    QStringList getFilePaths();

    QString getUserName();
    //获得系统版本
    QString getSysVer();
   // 获得系统架构
    QString getSysArch();
    //获得激活状态
    quint32 getActiveState();
    //获得激活码
    QString getActiveCode();
    //重置和发送成功，清除发送的内容
    void clearSendContent();

    void initUI();

    int readLog();

    DFrame* createFileFrame(QString );
};


#endif // MESSAGECONSULTATION_H
